@@ -7,7 +7,7 @@ Interface summary:
         x = copy.copy(y)        # make a shallow copy of y
         x = copy.deepcopy(y)    # make a deep copy of y
 
-For module specific errors, copy.error is raised.
+For module specific errors, copy.Error is raised.
 
 The difference between shallow and deep copying is only relevant for
 compound objects (objects that contain other objects, like lists or
@@ -48,9 +48,8 @@ __getstate__() and __setstate__().  See the documentation for module
 "pickle" for information on these methods.
 """
 
-# XXX need to support copy_reg here too...
-
 import types
+from copy_reg import dispatch_table
 
 class Error(Exception):
     pass
@@ -61,7 +60,7 @@ try:
 except ImportError:
     PyStringMap = None
 
-__all__ = ["Error", "error", "copy", "deepcopy"]
+__all__ = ["Error", "copy", "deepcopy"]
 
 def copy(x):
     """Shallow copy operation on arbitrary Python objects.
@@ -69,70 +68,62 @@ def copy(x):
     See the module's __doc__ string for more info.
     """
 
-    try:
-        copierfunction = _copy_dispatch[type(x)]
-    except KeyError:
-        try:
-            copier = x.__copy__
-        except AttributeError:
-            try:
-                reductor = x.__reduce__
-            except AttributeError:
-                raise error, \
-                      "un(shallow)copyable object of type %s" % type(x)
-            else:
-                y = _reconstruct(x, reductor(), 0)
-        else:
-            y = copier()
+    cls = type(x)
+
+    copier = _copy_dispatch.get(cls)
+    if copier:
+        return copier(x)
+
+    copier = getattr(cls, "__copy__", None)
+    if copier:
+        return copier(x)
+
+    reductor = dispatch_table.get(cls)
+    if reductor:
+        rv = reductor(x)
     else:
-        y = copierfunction(x)
-    return y
+        reductor = getattr(x, "__reduce_ex__", None)
+        if reductor:
+            rv = reductor(2)
+        else:
+            reductor = getattr(x, "__reduce__", None)
+            if reductor:
+                rv = reductor()
+            else:
+                raise Error("un(shallow)copyable object of type %s" % cls)
+
+    return _reconstruct(x, rv, 0)
+
 
 _copy_dispatch = d = {}
 
-def _copy_atomic(x):
+def _copy_immutable(x):
     return x
-d[types.NoneType] = _copy_atomic
-d[types.IntType] = _copy_atomic
-d[types.LongType] = _copy_atomic
-d[types.FloatType] = _copy_atomic
-try:
-    d[types.ComplexType] = _copy_atomic
-except AttributeError:
-    pass
-d[types.StringType] = _copy_atomic
-try:
-    d[types.UnicodeType] = _copy_atomic
-except AttributeError:
-    pass
-try:
-    d[types.CodeType] = _copy_atomic
-except AttributeError:
-    pass
-d[types.TypeType] = _copy_atomic
-d[types.XRangeType] = _copy_atomic
-d[types.ClassType] = _copy_atomic
+for t in (types.NoneType, int, long, float, bool, str, tuple,
+          frozenset, type, xrange, types.ClassType,
+          types.BuiltinFunctionType):
+    d[t] = _copy_immutable
+for name in ("ComplexType", "UnicodeType", "CodeType"):
+    t = getattr(types, name, None)
+    if t is not None:
+        d[t] = _copy_immutable
 
-def _copy_list(x):
-    return x[:]
-d[types.ListType] = _copy_list
+def _copy_with_constructor(x):
+    return type(x)(x)
+for t in (list, dict, set):
+    d[t] = _copy_with_constructor
 
-def _copy_tuple(x):
-    return x[:]
-d[types.TupleType] = _copy_tuple
-
-def _copy_dict(x):
+def _copy_with_copy_method(x):
     return x.copy()
-d[types.DictionaryType] = _copy_dict
 if PyStringMap is not None:
-    d[PyStringMap] = _copy_dict
+    d[PyStringMap] = _copy_with_copy_method
 
 def _copy_inst(x):
     if hasattr(x, '__copy__'):
         return x.__copy__()
     if hasattr(x, '__getinitargs__'):
         args = x.__getinitargs__()
-        y = apply(x.__class__, args)
+        y = x.__class__(*args)
     else:
         y = _EmptyClass()
         y.__class__ = x.__class__
@@ -149,7 +140,7 @@ d[types.InstanceType] = _copy_inst
 
 del d
 
-def deepcopy(x, memo = None):
+def deepcopy(x, memo=None, _nil=[]):
     """Deep copy operation on arbitrary Python objects.
 
     See the module's __doc__ string for more info.
@@ -157,33 +148,45 @@ def deepcopy(x, memo = None):
 
     if memo is None:
         memo = {}
+
     d = id(x)
-    if memo.has_key(d):
-        return memo[d]
-    try:
-        copierfunction = _deepcopy_dispatch[type(x)]
-    except KeyError:
+    y = memo.get(d, _nil)
+    if y is not _nil:
+        return y
+
+    cls = type(x)
+
+    copier = _deepcopy_dispatch.get(cls)
+    if copier:
+        y = copier(x, memo)
+    else:
         try:
-            issc = issubclass(type(x), type)
-        except TypeError:
+            issc = issubclass(cls, type)
+        except TypeError: # cls is not a class (old Boost; see SF #502085)
             issc = 0
         if issc:
-            y = _deepcopy_dispatch[type](x, memo)
+            y = _deepcopy_atomic(x, memo)
         else:
-            try:
-                copier = x.__deepcopy__
-            except AttributeError:
-                try:
-                    reductor = x.__reduce__
-                except AttributeError:
-                    raise error, \
-                       "un-deep-copyable object of type %s" % type(x)
-                else:
-                    y = _reconstruct(x, reductor(), 1, memo)
-            else:
+            copier = getattr(x, "__deepcopy__", None)
+            if copier:
                 y = copier(memo)
-    else:
-        y = copierfunction(x, memo)
+            else:
+                reductor = dispatch_table.get(cls)
+                if reductor:
+                    rv = reductor(x)
+                else:
+                    reductor = getattr(x, "__reduce_ex__", None)
+                    if reductor:
+                        rv = reductor(2)
+                    else:
+                        reductor = getattr(x, "__reduce__", None)
+                        if reductor:
+                            rv = reductor()
+                        else:
+                            raise Error(
+                                "un(deep)copyable object of type %s" % cls)
+                y = _reconstruct(x, rv, 1, memo)
+
     memo[d] = y
     _keep_alive(x, memo) # Make sure x lives at least as long as d
     return y
@@ -196,6 +199,7 @@ d[types.NoneType] = _deepcopy_atomic
 d[types.IntType] = _deepcopy_atomic
 d[types.LongType] = _deepcopy_atomic
 d[types.FloatType] = _deepcopy_atomic
+d[types.BooleanType] = _deepcopy_atomic
 try:
     d[types.ComplexType] = _deepcopy_atomic
 except AttributeError:
@@ -211,6 +215,8 @@ except AttributeError:
     pass
 d[types.TypeType] = _deepcopy_atomic
 d[types.XRangeType] = _deepcopy_atomic
+d[types.ClassType] = _deepcopy_atomic
+d[types.BuiltinFunctionType] = _deepcopy_atomic
 
 def _deepcopy_list(x, memo):
     y = []
@@ -242,8 +248,8 @@ d[types.TupleType] = _deepcopy_tuple
 def _deepcopy_dict(x, memo):
     y = {}
     memo[id(x)] = y
-    for key in x.keys():
-        y[deepcopy(key, memo)] = deepcopy(x[key], memo)
+    for key, value in x.iteritems():
+        y[deepcopy(key, memo)] = deepcopy(value, memo)
     return y
 d[types.DictionaryType] = _deepcopy_dict
 if PyStringMap is not None:
@@ -271,7 +277,7 @@ def _deepcopy_inst(x, memo):
     if hasattr(x, '__getinitargs__'):
         args = x.__getinitargs__()
         args = deepcopy(args, memo)
-        y = apply(x.__class__, args)
+        y = x.__class__(*args)
     else:
         y = _EmptyClass()
         y.__class__ = x.__class__
@@ -295,22 +301,50 @@ def _reconstruct(x, info, deep, memo=None):
     if memo is None:
         memo = {}
     n = len(info)
-    assert n in (2, 3)
+    assert n in (2, 3, 4, 5)
     callable, args = info[:2]
     if n > 2:
         state = info[2]
     else:
         state = {}
+    if n > 3:
+        listiter = info[3]
+    else:
+        listiter = None
+    if n > 4:
+        dictiter = info[4]
+    else:
+        dictiter = None
     if deep:
         args = deepcopy(args, memo)
     y = callable(*args)
+    memo[id(x)] = y
+    if listiter is not None:
+        for item in listiter:
+            if deep:
+                item = deepcopy(item, memo)
+            y.append(item)
+    if dictiter is not None:
+        for key, value in dictiter:
+            if deep:
+                key = deepcopy(key, memo)
+                value = deepcopy(value, memo)
+            y[key] = value
     if state:
         if deep:
             state = deepcopy(state, memo)
         if hasattr(y, '__setstate__'):
             y.__setstate__(state)
         else:
-            y.__dict__.update(state)
+            if isinstance(state, tuple) and len(state) == 2:
+                state, slotstate = state
+            else:
+                slotstate = None
+            if state is not None:
+                y.__dict__.update(state)
+            if slotstate is not None:
+                for key, value in slotstate.iteritems():
+                    setattr(y, key, value)
     return y
 
 del d
@@ -344,9 +378,9 @@ def _test():
         def __getstate__(self):
             return {'a': self.a, 'arg': self.arg}
         def __setstate__(self, state):
-            for key in state.keys():
-                setattr(self, key, state[key])
-        def __deepcopy__(self, memo = None):
+            for key, value in state.iteritems():
+                setattr(self, key, value)
+        def __deepcopy__(self, memo=None):
             new = self.__class__(deepcopy(self.arg, memo))
             new.a = self.a
             return new
