@@ -8,9 +8,9 @@ for the Microsoft Visual Studio.
 # hacked by Robin Becker and Thomas Heller to do a better job of
 #   finding DevStudio (through the registry)
 
-# This module should be kept compatible with Python 1.5.2.
+# This module should be kept compatible with Python 2.1.
 
-__revision__ = "$Id: msvccompiler.py,v 1.57 2003/07/17 14:41:07 jhylton Exp $"
+__revision__ = "$Id: msvccompiler.py,v 1.64 2004/11/28 01:10:01 tim_one Exp $"
 
 import sys, os, string
 from distutils.errors import \
@@ -117,17 +117,22 @@ class MacroExpander:
             if d:
                 self.macros["$(%s)" % macro] = d[key]
                 break
-              
+
     def load_macros(self, version):
         vsbase = r"Software\Microsoft\VisualStudio\%0.1f" % version
         self.set_macro("VCInstallDir", vsbase + r"\Setup\VC", "productdir")
         self.set_macro("VSInstallDir", vsbase + r"\Setup\VS", "productdir")
         net = r"Software\Microsoft\.NETFramework"
         self.set_macro("FrameworkDir", net, "installroot")
-        if version > 7.0:
-            self.set_macro("FrameworkSDKDir", net, "sdkinstallrootv1.1")
-        else:
-            self.set_macro("FrameworkSDKDir", net, "sdkinstallroot")
+        try:
+            if version > 7.0:
+                self.set_macro("FrameworkSDKDir", net, "sdkinstallrootv1.1")
+            else:
+                self.set_macro("FrameworkSDKDir", net, "sdkinstallroot")
+        except KeyError, exc: #
+            raise DistutilsPlatformError, \
+                  ("The .NET Framework SDK needs to be installed before "
+                   "building extensions for Python.")
 
         p = r"Software\Microsoft\NET Framework Setup\Product"
         for base in HKEYS:
@@ -166,7 +171,7 @@ def get_build_version():
         return majorVersion + minorVersion
     # else we don't know what version of the compiler this is
     return None
-    
+
 
 class MSVCCompiler (CCompiler) :
     """Concrete class that implements an interface to Microsoft Visual C++,
@@ -237,9 +242,14 @@ class MSVCCompiler (CCompiler) :
                                       '/Z7', '/D_DEBUG']
 
         self.ldflags_shared = ['/DLL', '/nologo', '/INCREMENTAL:NO']
-        self.ldflags_shared_debug = [
-            '/DLL', '/nologo', '/INCREMENTAL:no', '/pdb:None', '/DEBUG'
-            ]
+        if self.__version >= 7:
+            self.ldflags_shared_debug = [
+                '/DLL', '/nologo', '/INCREMENTAL:no', '/DEBUG'
+                ]
+        else:
+            self.ldflags_shared_debug = [
+                '/DLL', '/nologo', '/INCREMENTAL:no', '/pdb:None', '/DEBUG'
+                ]
         self.ldflags_static = [ '/nologo']
 
 
@@ -291,7 +301,11 @@ class MSVCCompiler (CCompiler) :
         else:
             compile_opts.extend(self.compile_options)
 
-        for obj, (src, ext) in build.items():
+        for obj in objects:
+            try:
+                src, ext = build[obj]
+            except KeyError:
+                continue
             if debug:
                 # pass the full pathname to MSVC in debug mode,
                 # this allows the debugger to find the source file
@@ -521,7 +535,7 @@ class MSVCCompiler (CCompiler) :
                 return fn
 
         return exe
-    
+
     def get_msvc_paths(self, path, platform='x86'):
         """Get a list of devstudio directories (include, lib or path).
 
@@ -547,6 +561,16 @@ class MSVCCompiler (CCompiler) :
                     return string.split(self.__macros.sub(d[path]), ";")
                 else:
                     return string.split(d[path], ";")
+        # MSVC 6 seems to create the registry entries we need only when
+        # the GUI is run.
+        if self.__version == 6:
+            for base in HKEYS:
+                if read_values(base, r"%s\6.0" % self.__root) is not None:
+                    self.warn("It seems you have Visual Studio 6 installed, "
+                        "but the expected registry settings are not present.\n"
+                        "You must at least run the Visual Studio GUI once "
+                        "so that these entries are created.")
+                break
         return []
 
     def set_path_env_var(self, name):
@@ -562,4 +586,3 @@ class MSVCCompiler (CCompiler) :
             p = self.get_msvc_paths(name)
         if p:
             os.environ[name] = string.join(p, ';')
-
