@@ -7,10 +7,9 @@ available.
 
 Written by:   Fred L. Drake, Jr.
 Email:        <fdrake@acm.org>
-Initial date: 17-Dec-1998
 """
 
-__revision__ = "$Id: sysconfig.py,v 1.44.6.3 2002/11/26 09:42:57 mwh Exp $"
+__revision__ = "$Id: sysconfig.py,v 1.61 2004/10/13 15:54:16 anthonybaxter Exp $"
 
 import os
 import re
@@ -34,11 +33,14 @@ python_build = os.path.isfile(landmark)
 
 del argv0_path, landmark
 
-# set_python_build() was present in 2.2 and 2.2.1; it's not needed
-# any more, but so 3rd party build scripts don't break, we leave
-# a do-nothing version:
-def set_python_build():
-    pass
+
+def get_python_version ():
+    """Return a string containing the major and minor Python version,
+    leaving off the patchlevel.  Sample return values could be '1.5'
+    or '2.2'.
+    """
+    return sys.version[:3]
+
 
 def get_python_inc(plat_specific=0, prefix=None):
     """Return the directory containing installed Python header files.
@@ -67,6 +69,11 @@ def get_python_inc(plat_specific=0, prefix=None):
     elif os.name == "nt":
         return os.path.join(prefix, "include")
     elif os.name == "mac":
+        if plat_specific:
+            return os.path.join(prefix, "Mac", "Include")
+        else:
+            return os.path.join(prefix, "Include")
+    elif os.name == "os2":
         return os.path.join(prefix, "Include")
     else:
         raise DistutilsPlatformError(
@@ -93,7 +100,7 @@ def get_python_lib(plat_specific=0, standard_lib=0, prefix=None):
 
     if os.name == "posix":
         libpython = os.path.join(prefix,
-                                 "lib", "python" + sys.version[:3])
+                                 "lib", "python" + get_python_version())
         if standard_lib:
             return libpython
         else:
@@ -119,6 +126,13 @@ def get_python_lib(plat_specific=0, standard_lib=0, prefix=None):
                 return os.path.join(prefix, "Lib")
             else:
                 return os.path.join(prefix, "Lib", "site-packages")
+
+    elif os.name == "os2":
+        if standard_lib:
+            return os.path.join(PREFIX, "Lib")
+        else:
+            return os.path.join(PREFIX, "Lib", "site-packages")
+
     else:
         raise DistutilsPlatformError(
             "I don't know where Python installs its library "
@@ -132,14 +146,37 @@ def customize_compiler(compiler):
     varies across Unices and is stored in Python's Makefile.
     """
     if compiler.compiler_type == "unix":
-        (cc, opt, ccshared, ldshared, so_ext) = \
-            get_config_vars('CC', 'OPT', 'CCSHARED', 'LDSHARED', 'SO')
+        (cc, cxx, opt, basecflags, ccshared, ldshared, so_ext) = \
+            get_config_vars('CC', 'CXX', 'OPT', 'BASECFLAGS', 'CCSHARED', 'LDSHARED', 'SO')
+
+        if os.environ.has_key('CC'):
+            cc = os.environ['CC']
+        if os.environ.has_key('CXX'):
+            cxx = os.environ['CXX']
+        if os.environ.has_key('LDSHARED'):
+            ldshared = os.environ['LDSHARED']
+        if os.environ.has_key('CPP'):
+            cpp = os.environ['CPP']
+        else:
+            cpp = cc + " -E"           # not always
+        if os.environ.has_key('LDFLAGS'):
+            ldshared = ldshared + ' ' + os.environ['LDFLAGS']
+        if basecflags:
+            opt = basecflags + ' ' + opt
+        if os.environ.has_key('CFLAGS'):
+            opt = opt + ' ' + os.environ['CFLAGS']
+            ldshared = ldshared + ' ' + os.environ['CFLAGS']
+        if os.environ.has_key('CPPFLAGS'):
+            cpp = cpp + ' ' + os.environ['CPPFLAGS']
+            opt = opt + ' ' + os.environ['CPPFLAGS']
+            ldshared = ldshared + ' ' + os.environ['CPPFLAGS']
 
         cc_cmd = cc + ' ' + opt
         compiler.set_executables(
-            preprocessor=cc + " -E",    # not always!
+            preprocessor=cpp,
             compiler=cc_cmd,
             compiler_so=cc_cmd + ' ' + ccshared,
+            compiler_cxx=cxx,
             linker_so=ldshared,
             linker_exe=cc)
 
@@ -187,7 +224,7 @@ def parse_config_h(fp, g=None):
         m = define_rx.match(line)
         if m:
             n, v = m.group(1, 2)
-            try: v = string.atoi(v)
+            try: v = int(v)
             except ValueError: pass
             g[n] = v
         else:
@@ -229,7 +266,7 @@ def parse_makefile(fn, g=None):
             if "$" in v:
                 notdone[n] = v
             else:
-                try: v = string.atoi(v)
+                try: v = int(v)
                 except ValueError: pass
                 done[n] = v
 
@@ -246,7 +283,7 @@ def parse_makefile(fn, g=None):
                     if "$" in after:
                         notdone[name] = value
                     else:
-                        try: value = string.atoi(value)
+                        try: value = int(value)
                         except ValueError:
                             done[name] = string.strip(value)
                         else:
@@ -262,7 +299,7 @@ def parse_makefile(fn, g=None):
                     if "$" in after:
                         notdone[name] = value
                     else:
-                        try: value = string.atoi(value)
+                        try: value = int(value)
                         except ValueError:
                             done[name] = string.strip(value)
                         else:
@@ -297,7 +334,6 @@ def expand_makefile_vars(s, vars):
     while 1:
         m = _findvar1_rx.search(s) or _findvar2_rx.search(s)
         if m:
-            name = m.group(1)
             (beg, end) = m.span()
             s = s[0:beg] + vars.get(m.group(1)) + s[end:]
         else:
@@ -321,6 +357,18 @@ def _init_posix():
 
         raise DistutilsPlatformError(my_msg)
 
+    # On MacOSX we need to check the setting of the environment variable
+    # MACOSX_DEPLOYMENT_TARGET: configure bases some choices on it so
+    # it needs to be compatible.
+    # An alternative would be to force MACOSX_DEPLOYMENT_TARGET to be
+    # the same as during configure.
+    if sys.platform == 'darwin' and g.has_key('CONFIGURE_MACOSX_DEPLOYMENT_TARGET'):
+        cfg_target = g['CONFIGURE_MACOSX_DEPLOYMENT_TARGET']
+        cur_target = os.getenv('MACOSX_DEPLOYMENT_TARGET', '')
+        if cfg_target != cur_target:
+            my_msg = ('$MACOSX_DEPLOYMENT_TARGET mismatch: now "%s" but "%s" during configure'
+                % (cur_target, cfg_target))
+            raise DistutilsPlatformError(my_msg)
 
     # On AIX, there are wrong paths to the linker scripts in the Makefile
     # -- these paths are relative to the Python source, but when installed
@@ -344,8 +392,10 @@ def _init_posix():
             # relative to the srcdir, which after installation no longer makes
             # sense.
             python_lib = get_python_lib(standard_lib=1)
-            linkerscript_name = os.path.basename(string.split(g['LDSHARED'])[0])
-            linkerscript = os.path.join(python_lib, 'config', linkerscript_name)
+            linkerscript_path = string.split(g['LDSHARED'])[0]
+            linkerscript_name = os.path.basename(linkerscript_path)
+            linkerscript = os.path.join(python_lib, 'config',
+                                        linkerscript_name)
 
             # XXX this isn't the right place to do this: adding the Python
             # library to the link, if needed, should be in the "build_ext"
@@ -396,6 +446,25 @@ def _init_mac():
     g['install_lib'] = os.path.join(EXEC_PREFIX, "Lib")
     g['install_platlib'] = os.path.join(EXEC_PREFIX, "Mac", "Lib")
 
+    # These are used by the extension module build
+    g['srcdir'] = ':'
+    global _config_vars
+    _config_vars = g
+
+
+def _init_os2():
+    """Initialize the module as appropriate for OS/2"""
+    g = {}
+    # set basic install directories
+    g['LIBDEST'] = get_python_lib(plat_specific=0, standard_lib=1)
+    g['BINLIBDEST'] = get_python_lib(plat_specific=1, standard_lib=1)
+
+    # XXX hmmm.. a normal install puts include files here
+    g['INCLUDEPY'] = get_python_inc(plat_specific=0)
+
+    g['SO'] = '.pyd'
+    g['EXE'] = ".exe"
+
     global _config_vars
     _config_vars = g
 
