@@ -16,7 +16,12 @@ import exceptions
 import linecache
 from code import InteractiveInterpreter
 
-from Tkinter import *
+try:
+    from Tkinter import *
+except ImportError:
+    print>>sys.__stderr__, "** IDLE can't import Tkinter.  " \
+                           "Your Python may not be configured for Tk. **"
+    sys.exit(1)
 import tkMessageBox
 
 from EditorWindow import EditorWindow, fixwordbreaks
@@ -39,25 +44,45 @@ try:
 except ImportError:
     SIGTERM = 15
 
-# Change warnings module to write to sys.__stderr__
+# Override warnings module to write to warning_stream.  Initialize to send IDLE
+# internal warnings to the console.  ScriptBinding.check_syntax() will
+# temporarily redirect the stream to the shell window to display warnings when
+# checking user's code.
+global warning_stream
+warning_stream = sys.__stderr__
 try:
     import warnings
 except ImportError:
     pass
 else:
     def idle_showwarning(message, category, filename, lineno):
-        file = sys.__stderr__
-        file.write(warnings.formatwarning(message, category, filename, lineno))
+        file = warning_stream
+        try:
+            file.write(warnings.formatwarning(message, category, filename, lineno))
+        except IOError:
+            pass  ## file (probably __stderr__) is invalid, warning dropped.
     warnings.showwarning = idle_showwarning
+    def idle_formatwarning(message, category, filename, lineno):
+        """Format warnings the IDLE way"""
+        s = "\nWarning (from warnings module):\n"
+        s += '  File \"%s\", line %s\n' % (filename, lineno)
+        line = linecache.getline(filename, lineno).strip()
+        if line:
+            s += "    %s\n" % line
+        s += "%s: %s\n>>> " % (category.__name__, message)
+        return s
+    warnings.formatwarning = idle_formatwarning
 
-def extended_linecache_checkcache(orig_checkcache=linecache.checkcache):
+def extended_linecache_checkcache(filename=None,
+                                  orig_checkcache=linecache.checkcache):
     """Extend linecache.checkcache to preserve the <pyshell#...> entries
 
-    Rather than repeating the linecache code, patch it to save the pyshell#
-    entries, call the original linecache.checkcache(), and then restore the
-    saved entries.  Assigning the orig_checkcache keyword arg freezes its value
-    at definition time to the (original) method linecache.checkcache(), i.e.
-    makes orig_checkcache lexical.
+    Rather than repeating the linecache code, patch it to save the
+    <pyshell#...> entries, call the original linecache.checkcache()
+    (which destroys them), and then restore the saved entries.
+
+    orig_checkcache is bound at definition time to the original
+    method, allowing it to be patched.
 
     """
     cache = linecache.cache
@@ -73,7 +98,7 @@ linecache.checkcache = extended_linecache_checkcache
 
 
 class PyShellEditorWindow(EditorWindow):
-    "Regular text edit window when a shell is present"
+    "Regular text edit window in IDLE, supports breakpoints"
 
     def __init__(self, *args):
         self.breakpoints = []
@@ -235,18 +260,22 @@ class PyShellEditorWindow(EditorWindow):
 
 
 class PyShellFileList(FileList):
-    "Extend base class: file list when a shell is present"
+    "Extend base class: IDLE supports a shell and breakpoints"
 
+    # override FileList's class variable, instances return PyShellEditorWindow
+    # instead of EditorWindow when new edit windows are created.
     EditorWindow = PyShellEditorWindow
 
     pyshell = None
 
     def open_shell(self, event=None):
         if self.pyshell:
-            self.pyshell.wakeup()
+            self.pyshell.top.wakeup()
         else:
             self.pyshell = PyShell(self)
-            self.pyshell.begin()
+            if self.pyshell:
+                if not self.pyshell.begin():
+                    return None
         return self.pyshell
 
 
@@ -328,10 +357,10 @@ class ModifiedInterpreter(InteractiveInterpreter):
         del_exitf = idleConf.GetOption('main', 'General', 'delete-exitfunc',
                                        default=False, type='bool')
         if __name__ == 'idlelib.PyShell':
-            command = "__import__('idlelib.run').run.main(" + `del_exitf` +")"
+            command = "__import__('idlelib.run').run.main(%r)" % (del_exitf,)
         else:
-            command = "__import__('run').main(" + `del_exitf` + ")"
-        if sys.platform == 'win32' and ' ' in sys.executable:
+            command = "__import__('run').main(%r)" % (del_exitf,)
+        if sys.platform[:3] == 'win' and ' ' in sys.executable:
             # handle embedded space in path by quoting the argument
             decorated_exec = '"%s"' % sys.executable
         else:
@@ -339,6 +368,9 @@ class ModifiedInterpreter(InteractiveInterpreter):
         return [decorated_exec] + w + ["-c", command, str(self.port)]
 
     def start_subprocess(self):
+        # spawning first avoids passing a listening socket to the subprocess
+        self.spawn_subprocess()
+        #time.sleep(20) # test to simulate GUI not accepting connection
         addr = (LOCALHOST, self.port)
         # Idle starts listening for connection on localhost
         for i in range(3):
@@ -347,14 +379,17 @@ class ModifiedInterpreter(InteractiveInterpreter):
                 self.rpcclt = MyRPCClient(addr)
                 break
             except socket.error, err:
-                print>>sys.__stderr__,"IDLE socket error: " + err[1]\
-                                                    + ", retrying..."
+                pass
         else:
-            display_port_binding_error()
-            sys.exit()
-        self.spawn_subprocess()
+            self.display_port_binding_error()
+            return None
         # Accept the connection from the Python execution server
-        self.rpcclt.accept()
+        self.rpcclt.listening_sock.settimeout(10)
+        try:
+            self.rpcclt.accept()
+        except socket.timeout, err:
+            self.display_no_subprocess_error()
+            return None
         self.rpcclt.register("stdin", self.tkconsole)
         self.rpcclt.register("stdout", self.tkconsole.stdout)
         self.rpcclt.register("stderr", self.tkconsole.stderr)
@@ -363,10 +398,11 @@ class ModifiedInterpreter(InteractiveInterpreter):
         self.rpcclt.register("interp", self)
         self.transfer_path()
         self.poll_subprocess()
+        return self.rpcclt
 
     def restart_subprocess(self):
         if self.restarting:
-            return
+            return self.rpcclt
         self.restarting = True
         # close only the subprocess debugger
         debug = self.getdebugger()
@@ -383,7 +419,11 @@ class ModifiedInterpreter(InteractiveInterpreter):
         was_executing = console.executing
         console.executing = False
         self.spawn_subprocess()
-        self.rpcclt.accept()
+        try:
+            self.rpcclt.accept()
+        except socket.timeout, err:
+            self.display_no_subprocess_error()
+            return None
         self.transfer_path()
         # annotate restart in shell window and mark it
         console.text.delete("iomark", "end-1c")
@@ -402,6 +442,7 @@ class ModifiedInterpreter(InteractiveInterpreter):
             # reload remote debugger breakpoints for all PyShellEditWindows
             debug.load_breakpoints()
         self.restarting = False
+        return self.rpcclt
 
     def __request_interrupt(self):
         self.rpcclt.remotecall("exec", "interrupt_the_server", (), {})
@@ -410,7 +451,10 @@ class ModifiedInterpreter(InteractiveInterpreter):
         threading.Thread(target=self.__request_interrupt).start()
 
     def kill_subprocess(self):
-        self.rpcclt.close()
+        try:
+            self.rpcclt.close()
+        except AttributeError:  # no socket
+            pass
         self.unix_terminate()
         self.tkconsole.executing = False
         self.rpcclt = None
@@ -432,12 +476,12 @@ class ModifiedInterpreter(InteractiveInterpreter):
     def transfer_path(self):
         self.runcommand("""if 1:
         import sys as _sys
-        _sys.path = %s
+        _sys.path = %r
         del _sys
         _msg = 'Use File/Exit or your end-of-file key to quit IDLE'
         __builtins__.quit = __builtins__.exit = _msg
         del _msg
-        \n""" % `sys.path`)
+        \n""" % (sys.path,))
 
     active_seq = None
 
@@ -461,7 +505,7 @@ class ModifiedInterpreter(InteractiveInterpreter):
             console = self.tkconsole.console
             if how == "OK":
                 if what is not None:
-                    print >>console, `what`
+                    print >>console, repr(what)
             elif how == "EXCEPTION":
                 if self.tkconsole.getvar("<<toggle-jit-stack-viewer>>"):
                     self.remote_stack_viewer()
@@ -506,7 +550,9 @@ class ModifiedInterpreter(InteractiveInterpreter):
         item = RemoteObjectBrowser.StubObjectTreeItem(self.rpcclt, oid)
         from TreeWidget import ScrolledCanvas, TreeNode
         top = Toplevel(self.tkconsole.root)
-        sc = ScrolledCanvas(top, bg="white", highlightthickness=0)
+        theme = idleConf.GetOption('main','Theme','name')
+        background = idleConf.GetHighlight(theme, 'normal')['background']
+        sc = ScrolledCanvas(top, bg=background, highlightthickness=0)
         sc.frame.pack(expand=1, fill="both")
         node = TreeNode(sc.canvas, None, item)
         node.expand()
@@ -567,14 +613,14 @@ class ModifiedInterpreter(InteractiveInterpreter):
     def prepend_syspath(self, filename):
         "Prepend sys.path with file's directory if not already included"
         self.runcommand("""if 1:
-            _filename = %s
+            _filename = %r
             import sys as _sys
             from os.path import dirname as _dirname
             _dir = _dirname(_filename)
             if not _dir in _sys.path:
                 _sys.path.insert(0, _dir)
             del _filename, _sys, _dirname, _dir
-            \n""" % `filename`)
+            \n""" % (filename,))
 
     def showsyntaxerror(self, filename=None):
         """Extend base class method: Add Colorizing
@@ -633,13 +679,6 @@ class ModifiedInterpreter(InteractiveInterpreter):
             if key[:1] + key[-1:] != "<>":
                 del c[key]
 
-    def display_executing_dialog(self):
-        tkMessageBox.showerror(
-            "Already executing",
-            "The Python Shell window is already executing a command; "
-            "please wait until it is finished.",
-            master=self.tkconsole.text)
-
     def runcommand(self, code):
         "Run the code without invoking the debugger"
         # The code better not raise an exception!
@@ -690,6 +729,34 @@ class ModifiedInterpreter(InteractiveInterpreter):
         "Override base class method"
         self.tkconsole.stderr.write(s)
 
+    def display_port_binding_error(self):
+        tkMessageBox.showerror(
+            "Port Binding Error",
+            "IDLE can't bind TCP/IP port 8833, which is necessary to "
+            "communicate with its Python execution server.  Either "
+            "no networking is installed on this computer or another "
+            "process (another IDLE?) is using the port.  Run IDLE with the -n "
+            "command line switch to start without a subprocess and refer to "
+            "Help/IDLE Help 'Running without a subprocess' for further "
+            "details.",
+            master=self.tkconsole.text)
+
+    def display_no_subprocess_error(self):
+        tkMessageBox.showerror(
+            "Subprocess Startup Error",
+            "IDLE's subprocess didn't make connection.  Either IDLE can't "
+            "start a subprocess or personal firewall software is blocking "
+            "the connection.",
+            master=self.tkconsole.text)
+
+    def display_executing_dialog(self):
+        tkMessageBox.showerror(
+            "Already executing",
+            "The Python Shell window is already executing a command; "
+            "please wait until it is finished.",
+            master=self.tkconsole.text)
+
+
 class PyShell(OutputWindow):
 
     shell_title = "Python Shell"
@@ -739,7 +806,6 @@ class PyShell(OutputWindow):
         text.bind("<<end-of-file>>", self.eof_callback)
         text.bind("<<open-stack-viewer>>", self.open_stack_viewer)
         text.bind("<<toggle-debugger>>", self.toggle_debugger)
-        text.bind("<<open-python-shell>>", self.flist.open_shell)
         text.bind("<<toggle-jit-stack-viewer>>", self.toggle_jit_stack_viewer)
         if use_subprocess:
             text.bind("<<view-restart>>", self.view_restart_mark)
@@ -760,8 +826,9 @@ class PyShell(OutputWindow):
         self.history = self.History(self.text)
         #
         self.pollinterval = 50  # millisec
-        if use_subprocess:
-            self.interp.start_subprocess()
+
+    def get_standard_extension_names(self):
+        return idleConf.GetExtensions(shell_only=True)
 
     reading = False
     executing = False
@@ -769,6 +836,13 @@ class PyShell(OutputWindow):
     endoffile = False
     closing = False
 
+    def set_warning_stream(self, stream):
+        global warning_stream
+        warning_stream = stream
+
+    def get_warning_stream(self):
+        return warning_stream
+
     def toggle_debugger(self, event=None):
         if self.executing:
             tkMessageBox.showerror("Don't debug now",
@@ -882,6 +956,10 @@ class PyShell(OutputWindow):
         self.resetoutput()
         if use_subprocess:
             nosub = ''
+            client = self.interp.start_subprocess()
+            if not client:
+                self.close()
+                return False
         else:
             nosub = "==== No Subprocess ===="
         self.write("Python %s on %s\n%s\n%s\nIDLE %s      %s\n" %
@@ -889,11 +967,8 @@ class PyShell(OutputWindow):
                     self.firewallmessage, idlever.IDLE_VERSION, nosub))
         self.showprompt()
         import Tkinter
-        Tkinter._default_root = None
-
-    def interact(self):
-        self.begin()
-        self.top.mainloop()
+        Tkinter._default_root = None # 03Jan04 KBK What's this?
+        return True
 
     def readline(self):
         save = self.reading
@@ -1276,11 +1351,9 @@ def main():
                 flist.open(filename)
             if not args:
                 flist.new()
-        if enable_shell:
-            flist.open_shell()
-    elif enable_shell:
-        flist.pyshell = PyShell(flist)
-        flist.pyshell.begin()
+    if enable_shell:
+        if not flist.open_shell():
+            return # couldn't open shell
     shell = flist.pyshell
     # handle remaining options:
     if debug:
@@ -1290,12 +1363,12 @@ def main():
                    os.environ.get("PYTHONSTARTUP")
         if filename and os.path.isfile(filename):
             shell.interp.execfile(filename)
-    if cmd or script:
+    if shell and cmd or script:
         shell.interp.runcommand("""if 1:
             import sys as _sys
-            _sys.argv = %s
+            _sys.argv = %r
             del _sys
-            \n""" % `sys.argv`)
+            \n""" % (sys.argv,))
         if cmd:
             shell.interp.execsource(cmd)
         elif script:
@@ -1304,24 +1377,6 @@ def main():
     root.mainloop()
     root.destroy()
 
-
-def display_port_binding_error():
-    print """\
-\nIDLE cannot run.
-
-IDLE needs to use a specific TCP/IP port (8833) in order to communicate with
-its Python execution server.  IDLE is unable to bind to this port, and so
-cannot start. Here are some possible causes of this problem:
-
-  1. TCP/IP networking is not installed or not working on this computer
-  2. Another program (another IDLE?) is running that uses this port
-  3. Personal firewall software is preventing IDLE from using this port
-
-Run IDLE with the -n command line switch to start without a subprocess
-and refer to Help/IDLE Help "Running without a subprocess" for further
-details.
-"""
-
 if __name__ == "__main__":
     sys.modules['PyShell'] = sys.modules['__main__']
     main()
