@@ -58,7 +58,7 @@ def pickle_code(co):
 
 #  def pickle_function(fn):
 #      assert isinstance(fn, type.FunctionType)
-#      return `fn`
+#      return repr(fn)
 
 copy_reg.pickle(types.CodeType, pickle_code, unpickle_code)
 # copy_reg.pickle(types.FunctionType, pickle_function, unpickle_function)
@@ -170,7 +170,7 @@ class SocketIO:
         except TypeError:
             return ("ERROR", "Bad request format")
         if not self.objtable.has_key(oid):
-            return ("ERROR", "Unknown object id: %s" % `oid`)
+            return ("ERROR", "Unknown object id: %r" % (oid,))
         obj = self.objtable[oid]
         if methodname == "__methods__":
             methods = {}
@@ -181,7 +181,7 @@ class SocketIO:
             _getattributes(obj, attributes)
             return ("OK", attributes)
         if not hasattr(obj, methodname):
-            return ("ERROR", "Unsupported method name: %s" % `methodname`)
+            return ("ERROR", "Unsupported method name: %r" % (methodname,))
         method = getattr(obj, methodname)
         try:
             if how == 'CALL':
@@ -320,23 +320,20 @@ class SocketIO:
         self.debug("putmessage:%d:" % message[0])
         try:
             s = pickle.dumps(message)
-        except pickle.UnpicklingError:
-            print >>sys.__stderr__, "Cannot pickle:", `message`
+        except pickle.PicklingError:
+            print >>sys.__stderr__, "Cannot pickle:", repr(message)
             raise
         s = struct.pack("<i", len(s)) + s
         while len(s) > 0:
             try:
-                n = self.sock.send(s)
+                r, w, x = select.select([], [self.sock], [])
+                n = self.sock.send(s[:BUFSIZE])
             except (AttributeError, socket.error):
                 # socket was closed
                 raise IOError
             else:
                 s = s[n:]
 
-    def ioready(self, wait):
-        r, w, x = select.select([self.sock.fileno()], [], [], wait)
-        return len(r)
-
     buffer = ""
     bufneed = 4
     bufstate = 0 # meaning: 0 => reading count; 1 => reading data
@@ -344,7 +341,8 @@ class SocketIO:
     def pollpacket(self, wait):
         self._stage0()
         if len(self.buffer) < self.bufneed:
-            if not self.ioready(wait):
+            r, w, x = select.select([self.sock.fileno()], [], [], wait)
+            if len(r) == 0:
                 return None
             try:
                 s = self.sock.recv(BUFSIZE)
@@ -377,9 +375,9 @@ class SocketIO:
             return None
         try:
             message = pickle.loads(packet)
-        except:
+        except pickle.UnpicklingError:
             print >>sys.__stderr__, "-----------------------"
-            print >>sys.__stderr__, "cannot unpickle packet:", `packet`
+            print >>sys.__stderr__, "cannot unpickle packet:", repr(packet)
             traceback.print_stack(file=sys.__stderr__)
             print >>sys.__stderr__, "-----------------------"
             raise
@@ -592,76 +590,6 @@ class MethodProxy:
         value = self.sockio.remotecall(self.oid, self.name, args, kwargs)
         return value
 
-#
-# Self Test
-#
 
-def testServer(addr):
-    # XXX 25 Jul 02 KBK needs update to use rpc.py register/unregister methods
-    class RemotePerson:
-        def __init__(self,name):
-            self.name = name
-        def greet(self, name):
-            print "(someone called greet)"
-            print "Hello %s, I am %s." % (name, self.name)
-            print
-        def getName(self):
-            print "(someone called getName)"
-            print
-            return self.name
-        def greet_this_guy(self, name):
-            print "(someone called greet_this_guy)"
-            print "About to greet %s ..." % name
-            remote_guy = self.server.current_handler.get_remote_proxy(name)
-            remote_guy.greet("Thomas Edison")
-            print "Done."
-            print
-
-    person = RemotePerson("Thomas Edison")
-    svr = RPCServer(addr)
-    svr.register('thomas', person)
-    person.server = svr # only required if callbacks are used
-
-    # svr.serve_forever()
-    svr.handle_request()  # process once only
-
-def testClient(addr):
-    "demonstrates RPC Client"
-    # XXX 25 Jul 02 KBK needs update to use rpc.py register/unregister methods
-    import time
-    clt=RPCClient(addr)
-    thomas = clt.get_remote_proxy("thomas")
-    print "The remote person's name is ..."
-    print thomas.getName()
-    # print clt.remotecall("thomas", "getName", (), {})
-    print
-    time.sleep(1)
-    print "Getting remote thomas to say hi..."
-    thomas.greet("Alexander Bell")
-    #clt.remotecall("thomas","greet",("Alexander Bell",), {})
-    print "Done."
-    print
-    time.sleep(2)
-    # demonstrates remote server calling local instance
-    class LocalPerson:
-        def __init__(self,name):
-            self.name = name
-        def greet(self, name):
-            print "You've greeted me!"
-        def getName(self):
-            return self.name
-    person = LocalPerson("Alexander Bell")
-    clt.register("alexander",person)
-    thomas.greet_this_guy("alexander")
-    # clt.remotecall("thomas","greet_this_guy",("alexander",), {})
-
-def test():
-    addr=(LOCALHOST, 8833)
-    if len(sys.argv) == 2:
-        if sys.argv[1]=='-server':
-            testServer(addr)
-            return
-    testClient(addr)
-
-if __name__ == '__main__':
-    test()
+# XXX KBK 09Sep03  We need a proper unit test for this module.  Previously
+#                  existing test code was removed at Rev 1.27.
