@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-""" This module tries to retrieve as much platform identifying data as
+""" This module tries to retrieve as much platform-identifying data as
     possible. It makes this information available via function APIs.
 
     If called from the command line, it prints the platform
@@ -20,7 +20,7 @@
 #    * support for Amiga and other still unsupported platforms running Python
 #    * support for additional Linux distributions
 #
-#    Many thanks to all those who helped adding platform specific
+#    Many thanks to all those who helped adding platform-specific
 #    checks (in no particular order):
 #
 #      Charles G Waldman, David Arnold, Gordon McMillan, Ben Darnell,
@@ -28,9 +28,11 @@
 #      Betancourt, Randall Hopper, Karl Putland, John Farrell, Greg
 #      Andruk, Just van Rossum, Thomas Heller, Mark R. Levinson, Mark
 #      Hammond, Bill Tutt, Hans Nowak, Uwe Zessin (OpenVMS support),
-#      Colin Kong, Trent Mick
+#      Colin Kong, Trent Mick, Guido van Rossum
 #
 #    History:
+#    1.0.3 - added normalization of Windows system name
+#    1.0.2 - added more Windows support
 #    1.0.1 - reformatted to make doc.py happy
 #    1.0.0 - reformatted a bit and checked into Python CVS
 #    0.8.0 - added sys.version parser and various new access
@@ -102,7 +104,7 @@ __copyright__ = """
 
 """
 
-__version__ = '1.0.1'
+__version__ = '1.0.2'
 
 import sys,string,os,re
 
@@ -118,15 +120,15 @@ def libc_ver(executable=sys.executable,lib='',version='',
 
              chunksize=2048):
 
-    """ Tries to determine the libc version against which the
-        file executable (defaults to the Python interpreter) is linked.
+    """ Tries to determine the libc version that the file executable
+        (which defaults to the Python interpreter) is linked against.
 
         Returns a tuple of strings (lib,version) which default to the
         given parameters in case the lookup fails.
 
         Note that the function has intimate knowledge of how different
-        libc versions add symbols to the executable is probably only
-        useable for executables compiled using gcc.
+        libc versions add symbols to the executable and thus is probably
+        only useable for executables compiled using gcc.
 
         The file is read and scanned in chunks of chunksize bytes.
 
@@ -219,13 +221,13 @@ def dist(distname='',version='',id='',
 
          supported_dists=('SuSE','debian','redhat','mandrake')):
 
-    """ Tries to determine the name of the OS distribution name
+    """ Tries to determine the name of the Linux OS distribution name.
 
         The function first looks for a distribution release file in
         /etc and then reverts to _dist_try_harder() in case no
         suitable files are found.
 
-        Returns a tuple distname,version,id which default to the
+        Returns a tuple (distname,version,id) which default to the
         args given as parameters.
 
     """
@@ -351,7 +353,7 @@ def popen(cmd, mode='r', bufsize=None):
 
 def _norm_version(version,build=''):
 
-    """ Normalize the version and build strings and return a sinlge
+    """ Normalize the version and build strings and return a single
         vesion string using the format major.minor.build (or patchlevel).
     """
     l = string.split(version,'.')
@@ -456,7 +458,11 @@ def win32_ver(release='',version='',csd='',ptype=''):
     """
     # XXX Is there any way to find out the processor type on WinXX ?
     # XXX Is win32 available on Windows CE ?
+    #
     # Adapted from code posted by Karl Putland to comp.lang.python.
+    #
+    # The mappings between reg. values and release names can be found
+    # here: http://msdn.microsoft.com/library/en-us/sysinfo/base/osversioninfo_str.asp
 
     # Import the needed APIs
     try:
@@ -478,8 +484,12 @@ def win32_ver(release='',version='',csd='',ptype=''):
         if maj == 4:
             if min == 0:
                 release = '95'
-            else:
+            elif min == 10:
                 release = '98'
+            elif min == 90:
+                release = 'Me'
+            else:
+                release = 'postMe'
         elif maj == 5:
             release = '2000'
     elif plat == VER_PLATFORM_WIN32_NT:
@@ -487,7 +497,14 @@ def win32_ver(release='',version='',csd='',ptype=''):
         if maj <= 4:
             release = 'NT'
         elif maj == 5:
-            release = '2000'
+            if min == 0:
+                release = '2000'
+            elif min == 1:
+                release = 'XP'
+            elif min == 2:
+                release = '2003Server'
+            else:
+                release = 'post2003'
     else:
         if not release:
             # E.g. Win3.1 with win32s
@@ -525,12 +542,13 @@ def win32_ver(release='',version='',csd='',ptype=''):
 def _mac_ver_lookup(selectors,default=None):
 
     from gestalt import gestalt
+    import MacOS
     l = []
     append = l.append
     for selector in selectors:
         try:
             append(gestalt(selector))
-        except RuntimeError:
+        except (RuntimeError, MacOS.Error):
             append(default)
     return l
 
@@ -544,8 +562,8 @@ def mac_ver(release='',versioninfo=('','',''),machine=''):
         versioninfo, machine) with versioninfo being a tuple (version,
         dev_stage, non_release_version).
 
-        Entries which cannot be determined are set to ''. All tuple
-        entries are strings.
+        Entries which cannot be determined are set to the paramter values
+        which default to ''. All tuple entries are strings.
 
         Thanks to Mark R. Levinson for mailing documentation links and
         code examples for this function. Documentation for the
@@ -557,6 +575,7 @@ def mac_ver(release='',versioninfo=('','',''),machine=''):
     # Check whether the version info module is available
     try:
         import gestalt
+        import MacOS
     except ImportError:
         return release,versioninfo,machine
     # Get the infos
@@ -595,7 +614,7 @@ def _java_getprop(name,default):
 
 def java_ver(release='',vendor='',vminfo=('','',''),osinfo=('','','')):
 
-    """ Version interface for JPython.
+    """ Version interface for Jython.
 
         Returns a tuple (release,vendor,vminfo,osinfo) with vminfo being
         a tuple (vm_name,vm_release,vm_vendor) and osinfo being a
@@ -815,9 +834,9 @@ _architecture_split = re.compile(r'[\s,]').split
 def architecture(executable=sys.executable,bits='',linkage=''):
 
     """ Queries the given executable (defaults to the Python interpreter
-        binary) for various architecture informations.
+        binary) for various architecture information.
 
-        Returns a tuple (bits,linkage) which contain information about
+        Returns a tuple (bits,linkage) which contains information about
         the bit architecture and the linkage format used for the
         executable. Both values are returned as strings.
 
@@ -828,9 +847,9 @@ def architecture(executable=sys.executable,bits='',linkage=''):
 
         The function relies on the system's "file" command to do the
         actual work. This is available on most if not all Unix
-        platforms. On some non-Unix platforms and then only if the
-        executable points to the Python interpreter defaults from
-        _default_architecture are used.
+        platforms. On some non-Unix platforms where the "file" command
+        does not exist and the executable is set to the Python interpreter
+        binary defaults from _default_architecture are used.
 
     """
     # Use the sizeof(pointer) as default number of bits if nothing
@@ -905,7 +924,7 @@ def uname():
         identifying the underlying platform.
 
         Note that unlike the os.uname function this also returns
-        possible processor information as additional tuple entry.
+        possible processor information as an additional tuple entry.
 
         Entries which cannot be determined are set to ''.
 
@@ -939,6 +958,10 @@ def uname():
         # platforms
         if use_syscmd_ver:
             system,release,version = _syscmd_ver(system)
+            # Normalize system to what win32_ver() normally returns
+            # (_syscmd_ver() tends to return the vendor name as well)
+            if system == 'Microsoft Windows':
+                system = 'Windows'
 
         # In case we still don't know anything useful, we'll try to
         # help ourselves
@@ -1013,7 +1036,8 @@ def system():
 
 def node():
 
-    """ Returns the computer's network name (may not be fully qualified !)
+    """ Returns the computer's network name (which may not be fully
+        qualified)
 
         An empty string is returned if the value cannot be determined.
 
@@ -1079,7 +1103,6 @@ def _sys_version():
 
     """
     global _sys_version_cache
-    import time
 
     if _sys_version_cache is not None:
         return _sys_version_cache
@@ -1133,8 +1156,7 @@ def python_compiler():
 
 ### The Opus Magnum of platform strings :-)
 
-_platform_cache = None
-_platform_aliased_cache = None
+_platform_cache = {}
 
 def platform(aliased=0, terse=0):
 
@@ -1155,12 +1177,9 @@ def platform(aliased=0, terse=0):
         absolute minimum information needed to identify the platform.
 
     """
-    global _platform_cache,_platform_aliased_cache
-
-    if not aliased and (_platform_cache is not None):
-        return _platform_cache
-    elif _platform_aliased_cache is not None:
-        return _platform_aliased_cache
+    result = _platform_cache.get((aliased, terse), None)
+    if result is not None:
+        return result
 
     # Get uname information and then apply platform specific cosmetics
     # to it...
@@ -1216,12 +1235,7 @@ def platform(aliased=0, terse=0):
             bits,linkage = architecture(sys.executable)
             platform = _platform(system,release,machine,processor,bits,linkage)
 
-    if aliased:
-        _platform_aliased_cache = platform
-    elif terse:
-        pass
-    else:
-        _platform_cache = platform
+    _platform_cache[(aliased, terse)] = platform
     return platform
 
 ### Command line interface
