@@ -1,8 +1,13 @@
 # Simple test suite for Cookie.py
 
-from test_support import verify, verbose, run_doctest
+from test.test_support import verify, verbose, run_doctest
 import Cookie
 
+import warnings
+warnings.filterwarnings("ignore",
+                        ".* class is insecure.*",
+                        DeprecationWarning)
+
 # Currently this only tests SimpleCookie
 
 cases = [
@@ -18,9 +23,7 @@ for data, dict in cases:
     C = Cookie.SimpleCookie() ; C.load(data)
     print repr(C)
     print str(C)
-    items = dict.items()
-    items.sort()
-    for k, v in items:
+    for k, v in sorted(dict.iteritems()):
         print ' ', k, repr( C[k].value ), repr(v)
         verify(C[k].value == v)
         print C[k]
