@@ -1,5 +1,5 @@
-# regression test for SAX 2.0
-# $Id: test_sax.py,v 1.19 2001/10/24 20:32:02 gvanrossum Exp $
+# regression test for SAX 2.0            -*- coding: iso-8859-1 -*-
+# $Id: test_sax.py,v 1.27 2004/08/03 10:17:34 mwh Exp $
 
 from xml.sax import make_parser, ContentHandler, \
                     SAXException, SAXReaderNotAvailable, SAXParseException
@@ -8,27 +8,28 @@ try:
 except SAXReaderNotAvailable:
     # don't try to test this module if we cannot create a parser
     raise ImportError("no XML parsers available")
-from xml.sax.saxutils import XMLGenerator, escape, quoteattr, XMLFilterBase
+from xml.sax.saxutils import XMLGenerator, escape, unescape, quoteattr, \
+                             XMLFilterBase
 from xml.sax.expatreader import create_parser
 from xml.sax.xmlreader import InputSource, AttributesImpl, AttributesNSImpl
 from cStringIO import StringIO
-from test_support import verify, verbose, TestFailed, findfile
+from test.test_support import verify, verbose, TestFailed, findfile
 import os
 
 # ===== Utilities
 
 tests = 0
-fails = 0
+failures = []
 
 def confirm(outcome, name):
-    global tests, fails
+    global tests
 
     tests = tests + 1
     if outcome:
-        print "Passed", name
+        if verbose:
+            print "Passed", name
     else:
-        print "Failed", name
-        fails = fails + 1
+        failures.append(name)
 
 def test_make_parser2():
     try:
@@ -70,6 +71,20 @@ def test_escape_all():
 def test_escape_extra():
     return escape("Hei p deg", {"" : "&aring;"}) == "Hei p&aring; deg"
 
+# ===== unescape
+
+def test_unescape_basic():
+    return unescape("Donald Duck &amp; Co") == "Donald Duck & Co"
+
+def test_unescape_all():
+    return unescape("&lt;Donald Duck &amp; Co&gt;") == "<Donald Duck & Co>"
+
+def test_unescape_extra():
+    return unescape("Hei p deg", {"" : "&aring;"}) == "Hei p&aring; deg"
+
+def test_unescape_amp_extra():
+    return unescape("&amp;foo;", {"&foo;": "splat"}) == "&foo;"
+
 # ===== quoteattr
 
 def test_quoteattr_basic():
@@ -337,7 +352,7 @@ def test_expat_nsattrs_wattr():
 
     return attrs.getLength() == 1 and \
            attrs.getNames() == [(ns_uri, "attr")] and \
-           attrs.getQNames() == [] and \
+           (attrs.getQNames() == [] or attrs.getQNames() == ["ns:attr"]) and \
            len(attrs) == 1 and \
            attrs.has_key((ns_uri, "attr")) and \
            attrs.keys() == [(ns_uri, "attr")] and \
@@ -474,6 +489,41 @@ def test_expat_incomplete():
     else:
         return 0
 
+def test_sax_parse_exception_str():
+    # pass various values from a locator to the SAXParseException to
+    # make sure that the __str__() doesn't fall apart when None is
+    # passed instead of an integer line and column number
+    #
+    # use "normal" values for the locator:
+    str(SAXParseException("message", None,
+                          DummyLocator(1, 1)))
+    # use None for the line number:
+    str(SAXParseException("message", None,
+                          DummyLocator(None, 1)))
+    # use None for the column number:
+    str(SAXParseException("message", None,
+                          DummyLocator(1, None)))
+    # use None for both:
+    str(SAXParseException("message", None,
+                          DummyLocator(None, None)))
+    return 1
+
+class DummyLocator:
+    def __init__(self, lineno, colno):
+        self._lineno = lineno
+        self._colno = colno
+
+    def getPublicId(self):
+        return "pubid"
+
+    def getSystemId(self):
+        return "sysid"
+
+    def getLineNumber(self):
+        return self._lineno
+
+    def getColumnNumber(self):
+        return self._colno
 
 # ===========================================================================
 #
@@ -637,7 +687,13 @@ items.sort()
 for (name, value) in items:
     if name[ : 5] == "test_":
         confirm(value(), name)
+# We delete the items variable so that the assignment to items above
+# doesn't pick up the old value of items (which messes with attempts
+# to find reference leaks).
+del items
 
-print "%d tests, %d failures" % (tests, fails)
-if fails != 0:
-    raise TestFailed, "%d of %d tests failed" % (fails, tests)
+if verbose:
+    print "%d tests, %d failures" % (tests, len(failures))
+if failures:
+    raise TestFailed("%d of %d tests failed: %s"
+                     % (len(failures), tests, ", ".join(failures)))
