@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-import unittest, operator, copy, pickle
+import unittest, operator, copy, pickle, random
 from sets import Set, ImmutableSet
 from test import test_support
 
@@ -12,7 +12,7 @@ class TestBasicOps(unittest.TestCase):
 
     def test_repr(self):
         if self.repr is not None:
-            self.assertEqual(`self.set`, self.repr)
+            self.assertEqual(repr(self.set), self.repr)
 
     def test_length(self):
         self.assertEqual(len(self.set), self.length)
@@ -596,6 +596,7 @@ class TestOnlySetsInBinaryOps(unittest.TestCase):
             self.set.difference(self.other)
         else:
             self.assertRaises(TypeError, self.set.difference, self.other)
+
 #------------------------------------------------------------------------------
 
 class TestOnlySetsNumeric(TestOnlySetsInBinaryOps):
@@ -647,6 +648,14 @@ class TestOnlySetsGenerator(TestOnlySetsInBinaryOps):
         self.other = gen()
         self.otherIsIterable = True
 
+#------------------------------------------------------------------------------
+
+class TestOnlySetsofSets(TestOnlySetsInBinaryOps):
+    def setUp(self):
+        self.set   = Set((1, 2, 3))
+        self.other = [Set('ab'), ImmutableSet('cd')]
+        self.otherIsIterable = True
+
 #==============================================================================
 
 class TestCopying(unittest.TestCase):
@@ -661,7 +670,7 @@ class TestCopying(unittest.TestCase):
 
     def test_deep_copy(self):
         dup = copy.deepcopy(self.set)
-        ##print type(dup), `dup`
+        ##print type(dup), repr(dup)
         dup_list = list(dup); dup_list.sort()
         set_list = list(self.set); set_list.sort()
         self.assertEqual(len(dup_list), len(set_list))
@@ -702,18 +711,18 @@ class TestCopyingNested(TestCopying):
 
 class TestIdentities(unittest.TestCase):
     def setUp(self):
-        self.a = Set('abracadabra')
-        self.b = Set('alacazam')
+        self.a = Set([random.randrange(100) for i in xrange(50)])
+        self.b = Set([random.randrange(100) for i in xrange(50)])
 
     def test_binopsVsSubsets(self):
         a, b = self.a, self.b
-        self.assert_(a - b < a)
-        self.assert_(b - a < b)
-        self.assert_(a & b < a)
-        self.assert_(a & b < b)
-        self.assert_(a | b > a)
-        self.assert_(a | b > b)
-        self.assert_(a ^ b < a | b)
+        self.assert_(a - b <= a)
+        self.assert_(b - a <= b)
+        self.assert_(a & b <= a)
+        self.assert_(a & b <= b)
+        self.assert_(a | b >= a)
+        self.assert_(a | b >= b)
+        self.assert_(a ^ b <= a | b)
 
     def test_commutativity(self):
         a, b = self.a, self.b
@@ -723,6 +732,16 @@ class TestIdentities(unittest.TestCase):
         if a != b:
             self.assertNotEqual(a-b, b-a)
 
+    def test_reflexsive_relations(self):
+        a, zero = self.a, Set()
+        self.assertEqual(a ^ a, zero)
+        self.assertEqual(a - a, zero)
+        self.assertEqual(a | a, a)
+        self.assertEqual(a & a, a)
+        self.assert_(a <= a)
+        self.assert_(a >= a)
+        self.assert_(a == a)
+
     def test_summations(self):
         # check that sums of parts equal the whole
         a, b = self.a, self.b
@@ -735,12 +754,20 @@ class TestIdentities(unittest.TestCase):
         self.assertEqual((a-b)|(b-a), a^b)
 
     def test_exclusion(self):
-        # check that inverse operations show non-overlap
+        # check that inverse operations do not overlap
         a, b, zero = self.a, self.b, Set()
         self.assertEqual((a-b)&b, zero)
         self.assertEqual((b-a)&a, zero)
         self.assertEqual((a&b)&(a^b), zero)
 
+    def test_cardinality_relations(self):
+        a, b = self.a, self.b
+        self.assertEqual(len(a), len(a-b) + len(a&b))
+        self.assertEqual(len(b), len(b-a) + len(a&b))
+        self.assertEqual(len(a^b), len(a-b) + len(b-a))
+        self.assertEqual(len(a|b), len(a-b) + len(a&b) + len(b-a))
+        self.assertEqual(len(a^b) + len(a&b), len(a|b))
+
 #==============================================================================
 
 libreftest = """
@@ -779,7 +806,7 @@ Set(['Jack', 'Jane', 'Janice', 'John', 'Marvin', 'Sam', 'Zack'])
 __test__ = {'libreftest' : libreftest}
 
 def test_main(verbose=None):
-    from test import test_sets
+    import test_sets, doctest
     test_support.run_unittest(
         TestSetOfSets,
         TestExceptionPropagation,
@@ -801,14 +828,15 @@ def test_main(verbose=None):
         TestOnlySetsTuple,
         TestOnlySetsString,
         TestOnlySetsGenerator,
+        TestOnlySetsofSets,
         TestCopyingEmpty,
         TestCopyingSingleton,
         TestCopyingTriple,
         TestCopyingTuple,
         TestCopyingNested,
         TestIdentities,
+        doctest.DocTestSuite(test_sets),
     )
-    test_support.run_doctest(test_sets, verbose)
 
 if __name__ == "__main__":
     test_main(verbose=True)
