@@ -31,7 +31,7 @@
  *   PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* CVS: $Id: _cursesmodule.c,v 2.71 2002/11/21 14:17:51 mwh Exp $ */
+/* CVS: $Id: _cursesmodule.c,v 2.73 2004/08/04 14:33:28 mwh Exp $ */
 
 /*
 
@@ -47,7 +47,7 @@ unsupported functions:
 	resizeterm restartterm ripoffline scr_dump
 	scr_init scr_restore scr_set scrl set_curterm set_term setterm
 	tgetent tgetflag tgetnum tgetstr tgoto timeout tputs
-	use_default_colors vidattr vidputs waddchnstr waddchstr wchgat
+	vidattr vidputs waddchnstr waddchstr wchgat
 	wcolor_set winchnstr winchstr winnstr wmouse_trafo wscrl
 
 Low-priority: 
@@ -2284,9 +2284,7 @@ PyCurses_TypeAhead(PyObject *self, PyObject *args)
 
   if (!PyArg_ParseTuple(args,"i;fd",&fd)) return NULL;
 
-  PyCursesCheckERR(typeahead( fd ), "typeahead");
-  Py_INCREF(Py_None);
-  return Py_None;
+  return PyCursesCheckERR(typeahead( fd ), "typeahead");
 }
 
 static PyObject *
@@ -2354,6 +2352,26 @@ PyCurses_Use_Env(PyObject *self, PyObject *args)
   return Py_None;
 }
 
+#ifndef STRICT_SYSV_CURSES
+static PyObject *
+PyCurses_Use_Default_Colors(PyObject *self)
+{
+  int code;
+
+  PyCursesInitialised
+  PyCursesInitialisedColor
+
+  code = use_default_colors();
+  if (code != ERR) {
+    Py_INCREF(Py_None);
+    return Py_None;
+  } else {
+    PyErr_SetString(PyCursesError, "use_default_colors() returned ERR");
+    return NULL;
+  }
+}
+#endif /* STRICT_SYSV_CURSES */
+
 /* List of functions defined in the module */
 
 static PyMethodDef PyCurses_methods[] = {
@@ -2434,6 +2452,9 @@ static PyMethodDef PyCurses_methods[] = {
   {"unctrl",              (PyCFunction)PyCurses_UnCtrl, METH_VARARGS},
   {"ungetch",             (PyCFunction)PyCurses_UngetCh, METH_VARARGS},
   {"use_env",             (PyCFunction)PyCurses_Use_Env, METH_VARARGS},
+#ifndef STRICT_SYSV_CURSES
+  {"use_default_colors",  (PyCFunction)PyCurses_Use_Default_Colors, METH_NOARGS},
+#endif
   {NULL,		  NULL}		/* sentinel */
 };
 
