@@ -3,12 +3,7 @@
 
 #include "Python.h"
 
-/* Mac with GUSI has more errors than those in errno.h */
-#ifdef USE_GUSI
-#include <sys/errno.h>
-#endif
-
-/* Windows socket errors (WSA*): XXX is this the correct path ???  */
+/* Windows socket errors (WSA*)  */
 #ifdef MS_WINDOWS
 #include <winsock.h>
 #endif
@@ -43,7 +38,7 @@ _inscode(PyObject *d, PyObject *de, char *name, int code)
 	Py_XDECREF(v);
 }
 
-static char errno__doc__ [] =
+PyDoc_STRVAR(errno__doc__,
 "This module makes available standard errno system symbols.\n\
 \n\
 The value of each symbol is the corresponding integer value,\n\
@@ -55,9 +50,9 @@ e.g., errno.errorcode[2] could be the string 'ENOENT'.\n\
 Symbols that are not relevant to the underlying system are not defined.\n\
 \n\
 To map error codes to error messages, use the function os.strerror(),\n\
-e.g. os.strerror(2) could return 'No such file or directory'.";
+e.g. os.strerror(2) could return 'No such file or directory'.");
 
-DL_EXPORT(void)
+PyMODINIT_FUNC
 initerrno(void)
 {
 	PyObject *m, *d, *de;
