@@ -1,7 +1,7 @@
 /*
  /  Author: Sam Rushing <rushing@nightmare.com>
  /  Hacked for Unix by AMK
- /  $Id: mmapmodule.c,v 2.46.6.1 2004/06/01 13:00:16 akuchling Exp $
+ /  $Id: mmapmodule.c,v 2.48 2004/05/19 14:39:08 akuchling Exp $
 
  / mmapmodule.cpp -- map a view of a file into memory
  /
@@ -39,11 +39,6 @@ my_getpagesize(void)
 #include <sys/mman.h>
 #include <sys/stat.h>
 
-#ifndef MS_SYNC
-/* This is missing e.g. on SunOS 4.1.4 */
-#define MS_SYNC 0
-#endif
-
 #if defined(HAVE_SYSCONF) && defined(_SC_PAGESIZE)
 static int
 my_getpagesize(void)
