@@ -33,7 +33,7 @@ extern void bzero(void *, int);
 #include <sys/types.h>
 #endif
 
-#if defined(PYOS_OS2)
+#if defined(PYOS_OS2) && !defined(PYCC_GCC)
 #include <sys/time.h>
 #include <utils.h>
 #endif
@@ -75,22 +75,29 @@ reap_obj(pylist fd2obj[FD_SETSIZE + 1])
    returns a number >= 0
 */
 static int
-list2set(PyObject *list, fd_set *set, pylist fd2obj[FD_SETSIZE + 1])
+seq2set(PyObject *seq, fd_set *set, pylist fd2obj[FD_SETSIZE + 1])
 {
 	int i;
 	int max = -1;
 	int index = 0;
-	int len = PyList_Size(list);
+        int len = -1;
+        PyObject* fast_seq = NULL;
 	PyObject* o = NULL;
 
 	fd2obj[0].obj = (PyObject*)0;	     /* set list to zero size */
 	FD_ZERO(set);
 
+        fast_seq=PySequence_Fast(seq, "arguments 1-3 must be sequences");
+        if (!fast_seq)
+            return -1;
+
+        len = PySequence_Fast_GET_SIZE(fast_seq);
+
 	for (i = 0; i < len; i++)  {
 		SOCKET v;
 
 		/* any intervening fileno() calls could decr this refcnt */
-		if (!(o = PyList_GetItem(list, i)))
+		if (!(o = PySequence_Fast_GET_ITEM(fast_seq, i)))
                     return -1;
 
 		Py_INCREF(o);
@@ -121,10 +128,12 @@ list2set(PyObject *list, fd_set *set, pylist fd2obj[FD_SETSIZE + 1])
 		fd2obj[index].sentinel = 0;
 		fd2obj[++index].sentinel = -1;
 	}
+        Py_DECREF(fast_seq);
 	return max+1;
 
   finally:
 	Py_XDECREF(o);
+        Py_DECREF(fast_seq);
 	return -1;
 }
 
@@ -208,12 +217,15 @@ select_select(PyObject *self, PyObject *args)
 
 	if (tout == Py_None)
 		tvp = (struct timeval *)0;
-	else if (!PyArg_Parse(tout, "d", &timeout)) {
+	else if (!PyNumber_Check(tout)) {
 		PyErr_SetString(PyExc_TypeError,
 				"timeout must be a float or None");
 		return NULL;
 	}
 	else {
+		timeout = PyFloat_AsDouble(tout);
+		if (timeout == -1 && PyErr_Occurred())
+			return NULL;
 		if (timeout > (double)LONG_MAX) {
 			PyErr_SetString(PyExc_OverflowError,
 					"timeout period too long");
@@ -226,15 +238,6 @@ select_select(PyObject *self, PyObject *args)
 		tvp = &tv;
 	}
 
-	/* sanity check first three arguments */
-	if (!PyList_Check(ifdlist) ||
-	    !PyList_Check(ofdlist) ||
-	    !PyList_Check(efdlist))
-	{
-		PyErr_SetString(PyExc_TypeError,
-				"arguments 1-3 must be lists");
-		return NULL;
-	}
 
 #ifdef SELECT_USES_HEAP
 	/* Allocate memory for the lists */
@@ -245,20 +248,20 @@ select_select(PyObject *self, PyObject *args)
 		if (rfd2obj) PyMem_DEL(rfd2obj);
 		if (wfd2obj) PyMem_DEL(wfd2obj);
 		if (efd2obj) PyMem_DEL(efd2obj);
-		return NULL;
+		return PyErr_NoMemory();
 	}
 #endif /* SELECT_USES_HEAP */
-	/* Convert lists to fd_sets, and get maximum fd number
-	 * propagates the Python exception set in list2set()
+	/* Convert sequences to fd_sets, and get maximum fd number
+	 * propagates the Python exception set in seq2set()
 	 */
 	rfd2obj[0].sentinel = -1;
 	wfd2obj[0].sentinel = -1;
 	efd2obj[0].sentinel = -1;
-	if ((imax=list2set(ifdlist, &ifdset, rfd2obj)) < 0) 
+	if ((imax=seq2set(ifdlist, &ifdset, rfd2obj)) < 0) 
 		goto finally;
-	if ((omax=list2set(ofdlist, &ofdset, wfd2obj)) < 0) 
+	if ((omax=seq2set(ofdlist, &ofdset, wfd2obj)) < 0) 
 		goto finally;
-	if ((emax=list2set(efdlist, &efdset, efd2obj)) < 0) 
+	if ((emax=seq2set(efdlist, &efdset, efd2obj)) < 0) 
 		goto finally;
 	max = imax;
 	if (omax > max) max = omax;
@@ -270,8 +273,7 @@ select_select(PyObject *self, PyObject *args)
 
 #ifdef MS_WINDOWS
 	if (n == SOCKET_ERROR) {
-		errno = WSAGetLastError();
-		PyErr_SetFromErrno(SelectError);
+		PyErr_SetExcFromWindowsErr(SelectError, WSAGetLastError());
 	}
 #else
 	if (n < 0) {
@@ -282,7 +284,7 @@ select_select(PyObject *self, PyObject *args)
                 /* optimization */
 		ifdlist = PyList_New(0);
 		if (ifdlist) {
-			ret = Py_BuildValue("OOO", ifdlist, ifdlist, ifdlist);
+			ret = PyTuple_Pack(3, ifdlist, ifdlist, ifdlist);
 			Py_DECREF(ifdlist);
 		}
 	}
@@ -297,7 +299,7 @@ select_select(PyObject *self, PyObject *args)
 		if (PyErr_Occurred())
 			ret = NULL;
 		else
-			ret = Py_BuildValue("OOO", ifdlist, ofdlist, efdlist);
+			ret = PyTuple_Pack(3, ifdlist, ofdlist, efdlist);
 
 		Py_DECREF(ifdlist);
 		Py_DECREF(ofdlist);
@@ -316,7 +318,7 @@ select_select(PyObject *self, PyObject *args)
 	return ret;
 }
 
-#ifdef HAVE_POLL
+#if defined(HAVE_POLL) && !defined(HAVE_BROKEN_POLL)
 /* 
  * poll() support
  */
@@ -329,7 +331,7 @@ typedef struct {
         struct pollfd *ufds;
 } pollObject;
 
-staticforward PyTypeObject poll_Type;
+static PyTypeObject poll_Type;
 
 /* Update the malloc'ed array of pollfds to match the dictionary 
    contained within a pollObject.  Return 1 on success, 0 on an error.
@@ -358,12 +360,12 @@ update_ufd_array(pollObject *self)
 	return 1;
 }
 
-static char poll_register_doc[] =
+PyDoc_STRVAR(poll_register_doc,
 "register(fd [, eventmask] ) -> None\n\n\
 Register a file descriptor with the polling object.\n\
 fd -- either an integer, or an object with a fileno() method returning an\n\
       int.\n\
-events -- an optional bitmask describing the type of events to check for";
+events -- an optional bitmask describing the type of events to check for");
 
 static PyObject *
 poll_register(pollObject *self, PyObject *args) 
@@ -401,9 +403,9 @@ poll_register(pollObject *self, PyObject *args)
 	return Py_None;
 }
 
-static char poll_unregister_doc[] =
+PyDoc_STRVAR(poll_unregister_doc,
 "unregister(fd) -> None\n\n\
-Remove a file descriptor being tracked by the polling object.";
+Remove a file descriptor being tracked by the polling object.");
 
 static PyObject *
 poll_unregister(pollObject *self, PyObject *args) 
@@ -438,10 +440,10 @@ poll_unregister(pollObject *self, PyObject *args)
 	return Py_None;
 }
 
-static char poll_poll_doc[] =
+PyDoc_STRVAR(poll_poll_doc,
 "poll( [timeout] ) -> list of (fd, event) 2-tuples\n\n\
 Polls the set of registered file descriptors, returning a list containing \n\
-any descriptors that have events or errors to report.";
+any descriptors that have events or errors to report.");
 
 static PyObject *
 poll_poll(pollObject *self, PyObject *args) 
@@ -457,11 +459,18 @@ poll_poll(pollObject *self, PyObject *args)
 	/* Check values for timeout */
 	if (tout == NULL || tout == Py_None)
 		timeout = -1;
-	else if (!PyArg_Parse(tout, "i", &timeout)) {
+	else if (!PyNumber_Check(tout)) {
 		PyErr_SetString(PyExc_TypeError,
 				"timeout must be an integer or None");
 		return NULL;
 	}
+	else {
+		tout = PyNumber_Int(tout);
+		if (!tout)
+			return NULL;
+		timeout = PyInt_AsLong(tout);
+		Py_DECREF(tout);
+	}
 
 	/* Ensure the ufd array is up to date */
 	if (!self->ufd_uptodate) 
@@ -502,7 +511,11 @@ poll_poll(pollObject *self, PyObject *args)
 			}
 			PyTuple_SET_ITEM(value, 0, num);
 
-			num = PyInt_FromLong(self->ufds[i].revents);
+			/* The &0xffff is a workaround for AIX.  'revents'
+			   is a 16-bit short, and IBM assigned POLLNVAL
+			   to be 0x8000, so the conversion to int results
+			   in a negative number. See SF bug #923315. */
+			num = PyInt_FromLong(self->ufds[i].revents & 0xffff);
 			if (num == NULL) {
 				Py_DECREF(value);
 				goto error;
@@ -566,7 +579,7 @@ poll_getattr(pollObject *self, char *name)
 	return Py_FindMethod(poll_methods, (PyObject *)self, name);
 }
 
-statichere PyTypeObject poll_Type = {
+static PyTypeObject poll_Type = {
 	/* The ob_type field must be initialized in the module init function
 	 * to be portable to Windows without using C++. */
 	PyObject_HEAD_INIT(NULL)
@@ -587,9 +600,9 @@ statichere PyTypeObject poll_Type = {
 	0,			/*tp_hash*/
 };
 
-static char poll_doc[] = 
+PyDoc_STRVAR(poll_doc,
 "Returns a polling object, which supports registering and\n\
-unregistering file descriptors, and then polling them for I/O events.";
+unregistering file descriptors, and then polling them for I/O events.");
 
 static PyObject *
 select_poll(PyObject *self, PyObject *args)
@@ -603,13 +616,13 @@ select_poll(PyObject *self, PyObject *args)
 		return NULL;
 	return (PyObject *)rv;
 }
-#endif /* HAVE_POLL */
+#endif /* HAVE_POLL && !HAVE_BROKEN_POLL */
 
-static char select_doc[] =
+PyDoc_STRVAR(select_doc,
 "select(rlist, wlist, xlist[, timeout]) -> (rlist, wlist, xlist)\n\
 \n\
 Wait until one or more file descriptors are ready for some kind of I/O.\n\
-The first three arguments are lists of file descriptors to be waited for:\n\
+The first three arguments are sequences of file descriptors to be waited for:\n\
 rlist -- wait until ready for reading\n\
 wlist -- wait until ready for writing\n\
 xlist -- wait for an ``exceptional condition''\n\
@@ -626,72 +639,54 @@ arguments; each contains the subset of the corresponding file descriptors\n\
 that are ready.\n\
 \n\
 *** IMPORTANT NOTICE ***\n\
-On Windows, only sockets are supported; on Unix, all file descriptors.";
+On Windows, only sockets are supported; on Unix, all file descriptors.");
 
 static PyMethodDef select_methods[] = {
     {"select",	select_select, METH_VARARGS, select_doc},
-#ifdef HAVE_POLL
+#if defined(HAVE_POLL) && !defined(HAVE_BROKEN_POLL)
     {"poll",    select_poll,   METH_VARARGS, poll_doc},
-#endif /* HAVE_POLL */
+#endif /* HAVE_POLL && !HAVE_BROKEN_POLL */
     {0,  	0},			     /* sentinel */
 };
 
-static char module_doc[] =
+PyDoc_STRVAR(module_doc,
 "This module supports asynchronous I/O on multiple file descriptors.\n\
 \n\
 *** IMPORTANT NOTICE ***\n\
-On Windows, only sockets are supported; on Unix, all file descriptors.";
+On Windows, only sockets are supported; on Unix, all file descriptors.");
 
-/*
- * Convenience routine to export an integer value.
- * For simplicity, errors (which are unlikely anyway) are ignored.
- */
-
-static void
-insint(PyObject *d, char *name, int value)
-{
-	PyObject *v = PyInt_FromLong((long) value);
-	if (v == NULL) {
-		/* Don't bother reporting this error */
-		PyErr_Clear();
-	}
-	else {
-		PyDict_SetItemString(d, name, v);
-		Py_DECREF(v);
-	}
-}
-
-DL_EXPORT(void)
+PyMODINIT_FUNC
 initselect(void)
 {
-	PyObject *m, *d;
+	PyObject *m;
 	m = Py_InitModule3("select", select_methods, module_doc);
-	d = PyModule_GetDict(m);
+
 	SelectError = PyErr_NewException("select.error", NULL, NULL);
-	PyDict_SetItemString(d, "error", SelectError);
-#ifdef HAVE_POLL
+	Py_INCREF(SelectError);
+	PyModule_AddObject(m, "error", SelectError);
+#if defined(HAVE_POLL) && !defined(HAVE_BROKEN_POLL)
 	poll_Type.ob_type = &PyType_Type;
-	insint(d, "POLLIN", POLLIN);
-	insint(d, "POLLPRI", POLLPRI);
-	insint(d, "POLLOUT", POLLOUT);
-	insint(d, "POLLERR", POLLERR);
-	insint(d, "POLLHUP", POLLHUP);
-	insint(d, "POLLNVAL", POLLNVAL);
+	PyModule_AddIntConstant(m, "POLLIN", POLLIN);
+	PyModule_AddIntConstant(m, "POLLPRI", POLLPRI);
+	PyModule_AddIntConstant(m, "POLLOUT", POLLOUT);
+	PyModule_AddIntConstant(m, "POLLERR", POLLERR);
+	PyModule_AddIntConstant(m, "POLLHUP", POLLHUP);
+	PyModule_AddIntConstant(m, "POLLNVAL", POLLNVAL);
 
 #ifdef POLLRDNORM
-	insint(d, "POLLRDNORM", POLLRDNORM);
+	PyModule_AddIntConstant(m, "POLLRDNORM", POLLRDNORM);
 #endif
 #ifdef POLLRDBAND
-	insint(d, "POLLRDBAND", POLLRDBAND);
+	PyModule_AddIntConstant(m, "POLLRDBAND", POLLRDBAND);
 #endif
 #ifdef POLLWRNORM
-	insint(d, "POLLWRNORM", POLLWRNORM);
+	PyModule_AddIntConstant(m, "POLLWRNORM", POLLWRNORM);
 #endif
 #ifdef POLLWRBAND
-	insint(d, "POLLWRBAND", POLLWRBAND);
+	PyModule_AddIntConstant(m, "POLLWRBAND", POLLWRBAND);
 #endif
 #ifdef POLLMSG
-	insint(d, "POLLMSG", POLLMSG);
+	PyModule_AddIntConstant(m, "POLLMSG", POLLMSG);
 #endif
-#endif /* HAVE_POLL */
+#endif /* HAVE_POLL && !HAVE_BROKEN_POLL */
 }
