@@ -1,6 +1,5 @@
 /* Abstract Object Interface (many thanks to Jim Fulton) */
 
-
 #include "Python.h"
 #include <ctype.h>
 #include "structmember.h" /* we need the offsetof() macro from there */
@@ -309,7 +308,9 @@ int PyObject_AsWriteBuffer(PyObject *obj,
 int
 PyNumber_Check(PyObject *o)
 {
-	return o && o->ob_type->tp_as_number;
+	return o && o->ob_type->tp_as_number &&
+	       (o->ob_type->tp_as_number->nb_int ||
+		o->ob_type->tp_as_number->nb_float);
 }
 
 /* Binary operators */
@@ -318,9 +319,9 @@ PyNumber_Check(PyObject *o)
 
 #define NB_SLOT(x) offsetof(PyNumberMethods, x)
 #define NB_BINOP(nb_methods, slot) \
-		((binaryfunc*)(& ((char*)nb_methods)[slot] ))
+		(*(binaryfunc*)(& ((char*)nb_methods)[slot]))
 #define NB_TERNOP(nb_methods, slot) \
-		((ternaryfunc*)(& ((char*)nb_methods)[slot] ))
+		(*(ternaryfunc*)(& ((char*)nb_methods)[slot]))
 
 /*
   Calling scheme used for binary operations:
@@ -352,10 +353,10 @@ binary_op1(PyObject *v, PyObject *w, const int op_slot)
 	binaryfunc slotw = NULL;
 
 	if (v->ob_type->tp_as_number != NULL && NEW_STYLE_NUMBER(v))
-		slotv = *NB_BINOP(v->ob_type->tp_as_number, op_slot);
+		slotv = NB_BINOP(v->ob_type->tp_as_number, op_slot);
 	if (w->ob_type != v->ob_type &&
 	    w->ob_type->tp_as_number != NULL && NEW_STYLE_NUMBER(w)) {
-		slotw = *NB_BINOP(w->ob_type->tp_as_number, op_slot);
+		slotw = NB_BINOP(w->ob_type->tp_as_number, op_slot);
 		if (slotw == slotv)
 			slotw = NULL;
 	}
@@ -387,7 +388,7 @@ binary_op1(PyObject *v, PyObject *w, const int op_slot)
 			PyNumberMethods *mv = v->ob_type->tp_as_number;
 			if (mv) {
 				binaryfunc slot;
-				slot = *NB_BINOP(mv, op_slot);
+				slot = NB_BINOP(mv, op_slot);
 				if (slot) {
 					PyObject *x = slot(v, w);
 					Py_DECREF(v);
@@ -403,20 +404,25 @@ binary_op1(PyObject *v, PyObject *w, const int op_slot)
 	Py_INCREF(Py_NotImplemented);
 	return Py_NotImplemented;
 }
-	    
+
+static PyObject *
+binop_type_error(PyObject *v, PyObject *w, const char *op_name)
+{
+	PyErr_Format(PyExc_TypeError,
+		     "unsupported operand type(s) for %s: '%s' and '%s'",
+		     op_name,
+		     v->ob_type->tp_name,
+		     w->ob_type->tp_name);
+	return NULL;
+}
+
 static PyObject *
 binary_op(PyObject *v, PyObject *w, const int op_slot, const char *op_name)
 {
 	PyObject *result = binary_op1(v, w, op_slot);
 	if (result == Py_NotImplemented) {
-		Py_DECREF(Py_NotImplemented);
-		PyErr_Format(
-			PyExc_TypeError, 
-			"unsupported operand type(s) for %s: '%s' and '%s'",
-			op_name,
-			v->ob_type->tp_name,
-			w->ob_type->tp_name);
-		return NULL;
+		Py_DECREF(result);
+		return binop_type_error(v, w, op_name);
 	}
 	return result;
 }
@@ -462,14 +468,14 @@ ternary_op(PyObject *v,
 	ternaryfunc slotv = NULL;
 	ternaryfunc slotw = NULL;
 	ternaryfunc slotz = NULL;
-	
+
 	mv = v->ob_type->tp_as_number;
 	mw = w->ob_type->tp_as_number;
 	if (mv != NULL && NEW_STYLE_NUMBER(v))
-		slotv = *NB_TERNOP(mv, op_slot);
+		slotv = NB_TERNOP(mv, op_slot);
 	if (w->ob_type != v->ob_type &&
-	    mv != NULL && NEW_STYLE_NUMBER(w)) {
-		slotw = *NB_TERNOP(mw, op_slot);
+	    mw != NULL && NEW_STYLE_NUMBER(w)) {
+		slotw = NB_TERNOP(mw, op_slot);
 		if (slotw == slotv)
 			slotw = NULL;
 	}
@@ -494,7 +500,7 @@ ternary_op(PyObject *v,
 	}
 	mz = z->ob_type->tp_as_number;
 	if (mz != NULL && NEW_STYLE_NUMBER(z)) {
-		slotz = *NB_TERNOP(mz, op_slot);
+		slotz = NB_TERNOP(mz, op_slot);
 		if (slotz == slotv || slotz == slotw)
 			slotz = NULL;
 		if (slotz) {
@@ -510,7 +516,7 @@ ternary_op(PyObject *v,
 		/* we have an old style operand, coerce */
 		PyObject *v1, *z1, *w2, *z2;
 		int c;
-		
+
 		c = PyNumber_Coerce(&v, &w);
 		if (c != 0)
 			goto error3;
@@ -519,8 +525,8 @@ ternary_op(PyObject *v,
 		   treated as absent argument and not coerced. */
 		if (z == Py_None) {
 			if (v->ob_type->tp_as_number) {
-				slotz = *NB_TERNOP(v->ob_type->tp_as_number,
-						   op_slot);
+				slotz = NB_TERNOP(v->ob_type->tp_as_number,
+						  op_slot);
 				if (slotz)
 					x = slotz(v, w, z);
 				else
@@ -542,8 +548,8 @@ ternary_op(PyObject *v,
 			goto error1;
 
 		if (v1->ob_type->tp_as_number != NULL) {
-			slotv = *NB_TERNOP(v1->ob_type->tp_as_number,
-					   op_slot);
+			slotv = NB_TERNOP(v1->ob_type->tp_as_number,
+					  op_slot);
 			if (slotv)
 				x = slotv(v1, w2, z2);
 			else
@@ -595,7 +601,6 @@ BINARY_FUNC(PyNumber_And, nb_and, "&")
 BINARY_FUNC(PyNumber_Lshift, nb_lshift, "<<")
 BINARY_FUNC(PyNumber_Rshift, nb_rshift, ">>")
 BINARY_FUNC(PyNumber_Subtract, nb_subtract, "-")
-BINARY_FUNC(PyNumber_Multiply, nb_multiply, "*")
 BINARY_FUNC(PyNumber_Divide, nb_divide, "/")
 BINARY_FUNC(PyNumber_Divmod, nb_divmod, "divmod()")
 
@@ -611,17 +616,75 @@ PyNumber_Add(PyObject *v, PyObject *w)
 		}
 		if (result == Py_NotImplemented) {
 			Py_DECREF(result);
-			PyErr_Format(
-			    PyExc_TypeError,
-			    "unsupported operand types for +: '%s' and '%s'",
-			    v->ob_type->tp_name,
-			    w->ob_type->tp_name);
-			result = NULL;
+			return binop_type_error(v, w, "+");
                 }
 	}
 	return result;
 }
 
+static PyObject *
+sequence_repeat(intargfunc repeatfunc, PyObject *seq, PyObject *n)
+{
+	long count;
+	if (PyInt_Check(n)) {
+		count  = PyInt_AsLong(n);
+	}
+	else if (PyLong_Check(n)) {
+		count = PyLong_AsLong(n);
+		if (count == -1 && PyErr_Occurred())
+			return NULL;
+	}
+	else {
+		return type_error(
+			"can't multiply sequence by non-int");
+	}
+#if LONG_MAX != INT_MAX
+	if (count > INT_MAX) {
+		PyErr_SetString(PyExc_ValueError,
+				"sequence repeat count too large");
+		return NULL;
+	}
+	else if (count < INT_MIN)
+		count = INT_MIN;
+	/* XXX Why don't I either
+
+	   - set count to -1 whenever it's negative (after all,
+	     sequence repeat usually treats negative numbers
+	     as zero(); or
+
+	   - raise an exception when it's less than INT_MIN?
+
+	   I'm thinking about a hypothetical use case where some
+	   sequence type might use a negative value as a flag of
+	   some kind.  In those cases I don't want to break the
+	   code by mapping all negative values to -1.  But I also
+	   don't want to break e.g. []*(-sys.maxint), which is
+	   perfectly safe, returning [].  As a compromise, I do
+	   map out-of-range negative values.
+	*/
+#endif
+	return (*repeatfunc)(seq, (int)count);
+}
+
+PyObject *
+PyNumber_Multiply(PyObject *v, PyObject *w)
+{
+	PyObject *result = binary_op1(v, w, NB_SLOT(nb_multiply));
+	if (result == Py_NotImplemented) {
+		PySequenceMethods *mv = v->ob_type->tp_as_sequence;
+		PySequenceMethods *mw = w->ob_type->tp_as_sequence;
+		Py_DECREF(result);
+		if  (mv && mv->sq_repeat) {
+			return sequence_repeat(mv->sq_repeat, v, w);
+		}
+		else if (mw && mw->sq_repeat) {
+			return sequence_repeat(mw->sq_repeat, w, v);
+		}
+		result = binop_type_error(v, w, "*");
+	}
+	return result;
+}
+
 PyObject *
 PyNumber_FloorDivide(PyObject *v, PyObject *w)
 {
@@ -639,12 +702,6 @@ PyNumber_TrueDivide(PyObject *v, PyObject *w)
 PyObject *
 PyNumber_Remainder(PyObject *v, PyObject *w)
 {
-	if (PyString_Check(v))
-		return PyString_Format(v, w);
-#ifdef Py_USING_UNICODE
-	else if (PyUnicode_Check(v))
-		return PyUnicode_Format(v, w);
-#endif
 	return binary_op(v, w, NB_SLOT(nb_remainder), "%");
 }
 
@@ -664,30 +721,42 @@ PyNumber_Power(PyObject *v, PyObject *w, PyObject *z)
      result.  No coercion is done on the arguments; the left-hand object
      is the one the operation is performed on, and it's up to the
      function to deal with the right-hand object.
-     
+
    - Otherwise, in-place modification is not supported. Handle it exactly as
      a non in-place operation of the same kind.
 
    */
 
-#define HASINPLACE(t) PyType_HasFeature((t)->ob_type, Py_TPFLAGS_HAVE_INPLACEOPS)
+#define HASINPLACE(t) \
+	PyType_HasFeature((t)->ob_type, Py_TPFLAGS_HAVE_INPLACEOPS)
 
 static PyObject *
-binary_iop(PyObject *v, PyObject *w, const int iop_slot, const int op_slot,
-		const char *op_name)
+binary_iop1(PyObject *v, PyObject *w, const int iop_slot, const int op_slot)
 {
 	PyNumberMethods *mv = v->ob_type->tp_as_number;
 	if (mv != NULL && HASINPLACE(v)) {
-		binaryfunc *slot = NB_BINOP(mv, iop_slot);
-		if (*slot) {
-			PyObject *x = (*slot)(v, w);
+		binaryfunc slot = NB_BINOP(mv, iop_slot);
+		if (slot) {
+			PyObject *x = (slot)(v, w);
 			if (x != Py_NotImplemented) {
 				return x;
 			}
 			Py_DECREF(x);
 		}
 	}
-	return binary_op(v, w, op_slot, op_name);
+	return binary_op1(v, w, op_slot);
+}
+
+static PyObject *
+binary_iop(PyObject *v, PyObject *w, const int iop_slot, const int op_slot,
+		const char *op_name)
+{
+	PyObject *result = binary_iop1(v, w, iop_slot, op_slot);
+	if (result == Py_NotImplemented) {
+		Py_DECREF(result);
+		return binop_type_error(v, w, op_name);
+	}
+	return result;
 }
 
 #define INPLACE_BINOP(func, iop, op, op_name) \
@@ -723,64 +792,62 @@ PyNumber_InPlaceTrueDivide(PyObject *v, PyObject *w)
 PyObject *
 PyNumber_InPlaceAdd(PyObject *v, PyObject *w)
 {
-	binaryfunc f = NULL;
-
-	if (v->ob_type->tp_as_sequence != NULL) {
-		if (HASINPLACE(v))
-			f = v->ob_type->tp_as_sequence->sq_inplace_concat;
-		if (f == NULL)
-			f = v->ob_type->tp_as_sequence->sq_concat;
-		if (f != NULL)
-			return (*f)(v, w);
+	PyObject *result = binary_iop1(v, w, NB_SLOT(nb_inplace_add),
+				       NB_SLOT(nb_add));
+	if (result == Py_NotImplemented) {
+		PySequenceMethods *m = v->ob_type->tp_as_sequence;
+		Py_DECREF(result);
+		if (m != NULL) {
+			binaryfunc f = NULL;
+			if (HASINPLACE(v))
+				f = m->sq_inplace_concat;
+			if (f == NULL)
+				f = m->sq_concat;
+			if (f != NULL)
+				return (*f)(v, w);
+		}
+		result = binop_type_error(v, w, "+=");
 	}
-	return binary_iop(v, w, NB_SLOT(nb_inplace_add), NB_SLOT(nb_add), "+=");
+	return result;
 }
 
 PyObject *
 PyNumber_InPlaceMultiply(PyObject *v, PyObject *w)
 {
-	PyObject * (*g)(PyObject *, int) = NULL;
-	if (HASINPLACE(v) &&
-	    v->ob_type->tp_as_sequence &&
-	    (g = v->ob_type->tp_as_sequence->sq_inplace_repeat) &&
-	    !(v->ob_type->tp_as_number &&
-	      v->ob_type->tp_as_number->nb_inplace_multiply))
-	{
-		long n;
-		if (PyInt_Check(w)) {
-			n  = PyInt_AsLong(w);
+	PyObject *result = binary_iop1(v, w, NB_SLOT(nb_inplace_multiply),
+				       NB_SLOT(nb_multiply));
+	if (result == Py_NotImplemented) {
+		intargfunc f = NULL;
+		PySequenceMethods *mv = v->ob_type->tp_as_sequence;
+		PySequenceMethods *mw = w->ob_type->tp_as_sequence;
+		Py_DECREF(result);
+		if (mv != NULL) {
+			if (HASINPLACE(v))
+				f = mv->sq_inplace_repeat;
+			if (f == NULL)
+				f = mv->sq_repeat;
+			if (f != NULL)
+				return sequence_repeat(f, v, w);
 		}
-		else if (PyLong_Check(w)) {
-			n = PyLong_AsLong(w);
-			if (n == -1 && PyErr_Occurred())
-				return NULL;
+		else if (mw != NULL) {
+			/* Note that the right hand operand should not be
+			 * mutated in this case so sq_inplace_repeat is not
+			 * used. */
+			if (mw->sq_repeat)
+				return sequence_repeat(mw->sq_repeat, w, v);
 		}
-		else {
-			return type_error("can't multiply sequence to non-int");
-		}
-		return (*g)(v, (int)n);
+		result = binop_type_error(v, w, "*=");
 	}
-	return binary_iop(v, w, NB_SLOT(nb_inplace_multiply),
-				NB_SLOT(nb_multiply), "*=");
+	return result;
 }
 
-
-
 PyObject *
 PyNumber_InPlaceRemainder(PyObject *v, PyObject *w)
 {
-	if (PyString_Check(v))
-		return PyString_Format(v, w);
-#ifdef Py_USING_UNICODE
-	else if (PyUnicode_Check(v))
-		return PyUnicode_Format(v, w);
-#endif
-	else
-		return binary_iop(v, w, NB_SLOT(nb_inplace_remainder),
-					NB_SLOT(nb_remainder), "%=");
+	return binary_iop(v, w, NB_SLOT(nb_inplace_remainder),
+				NB_SLOT(nb_remainder), "%=");
 }
 
-
 PyObject *
 PyNumber_InPlacePower(PyObject *v, PyObject *w, PyObject *z)
 {
@@ -889,7 +956,7 @@ PyNumber_Int(PyObject *o)
 		return PyInt_FromLong(io->ob_ival);
 	}
 	if (PyString_Check(o))
-		return int_from_string(PyString_AS_STRING(o), 
+		return int_from_string(PyString_AS_STRING(o),
 				       PyString_GET_SIZE(o));
 #ifdef Py_USING_UNICODE
 	if (PyUnicode_Check(o))
@@ -898,8 +965,17 @@ PyNumber_Int(PyObject *o)
 					 10);
 #endif
 	m = o->ob_type->tp_as_number;
-	if (m && m->nb_int)
-		return m->nb_int(o);
+	if (m && m->nb_int) {
+		PyObject *res = m->nb_int(o);
+		if (res && (!PyInt_Check(res) && !PyLong_Check(res))) {
+			PyErr_Format(PyExc_TypeError,
+				     "__int__ returned non-int (type %.200s)",
+				     res->ob_type->tp_name);
+			Py_DECREF(res);
+			return NULL;
+		}
+		return res;
+	}
 	if (!PyObject_AsCharBuffer(o, &buffer, &buffer_len))
 		return int_from_string((char*)buffer, buffer_len);
 
@@ -941,7 +1017,7 @@ PyNumber_Long(PyObject *o)
 	if (PyLong_Check(o))
 		return _PyLong_Copy((PyLongObject *)o);
 	if (PyString_Check(o))
-		/* need to do extra error checking that PyLong_FromString() 
+		/* need to do extra error checking that PyLong_FromString()
 		 * doesn't do.  In particular long('9.5') must raise an
 		 * exception, not truncate the float.
 		 */
@@ -955,8 +1031,17 @@ PyNumber_Long(PyObject *o)
 					  10);
 #endif
 	m = o->ob_type->tp_as_number;
-	if (m && m->nb_long)
-		return m->nb_long(o);
+	if (m && m->nb_long) {
+		PyObject *res = m->nb_long(o);
+		if (res && (!PyInt_Check(res) && !PyLong_Check(res))) {
+			PyErr_Format(PyExc_TypeError,
+				     "__long__ returned non-long (type %.200s)",
+				     res->ob_type->tp_name);
+			Py_DECREF(res);
+			return NULL;
+		}
+		return res;
+	}
 	if (!PyObject_AsCharBuffer(o, &buffer, &buffer_len))
 		return long_from_string(buffer, buffer_len);
 
@@ -980,8 +1065,17 @@ PyNumber_Float(PyObject *o)
 	}
 	if (!PyString_Check(o)) {
 		m = o->ob_type->tp_as_number;
-		if (m && m->nb_float)
-			return m->nb_float(o);
+		if (m && m->nb_float) {
+			PyObject *res = m->nb_float(o);
+			if (res && !PyFloat_Check(res)) {
+				PyErr_Format(PyExc_TypeError,
+			          "__float__ returned non-float (type %.200s)",
+			          res->ob_type->tp_name);
+				Py_DECREF(res);
+				return NULL;
+			}
+			return res;
+		}
 	}
 	return PyFloat_FromString(o, NULL);
 }
@@ -991,6 +1085,8 @@ PyNumber_Float(PyObject *o)
 int
 PySequence_Check(PyObject *s)
 {
+	if (s && PyInstance_Check(s))
+		return PyObject_HasAttrString(s, "__getitem__");
 	return s != NULL && s->ob_type->tp_as_sequence &&
 		s->ob_type->tp_as_sequence->sq_item != NULL;
 }
@@ -1185,7 +1281,7 @@ PySequence_SetItem(PyObject *s, int i, PyObject *o)
 		return m->sq_ass_item(s, i, o);
 	}
 
-	type_error("object doesn't support item assignment");
+	type_error("object does not support item assignment");
 	return -1;
 }
 
@@ -1313,7 +1409,7 @@ PySequence_Tuple(PyObject *v)
 		return NULL;
 
 	/* Guess result size and allocate space. */
-	n = PySequence_Size(v);
+	n = PyObject_Size(v);
 	if (n < 0) {
 		PyErr_Clear();
 		n = 10;  /* arbitrary */
@@ -1360,90 +1456,47 @@ Fail:
 PyObject *
 PySequence_List(PyObject *v)
 {
-	PyObject *it;      /* iter(v) */
 	PyObject *result;  /* result list */
-	int n;		   /* guess for result list size */
-	int i;
+	PyObject *rv;      /* return value from PyList_Extend */
 
 	if (v == NULL)
 		return null_error();
 
-	/* Special-case list(a_list), for speed. */
-	if (PyList_Check(v))
-		return PyList_GetSlice(v, 0, PyList_GET_SIZE(v));
-
-	/* Get iterator.  There may be some low-level efficiency to be gained
-	 * by caching the tp_iternext slot instead of using PyIter_Next()
-	 * later, but premature optimization is the root etc.
-	 */
-	it = PyObject_GetIter(v);
-	if (it == NULL)
+	result = PyList_New(0);
+	if (result == NULL)
 		return NULL;
 
-	/* Guess a result list size. */
-	n = -1;	 /* unknown */
-	if (PySequence_Check(v) &&
-	    v->ob_type->tp_as_sequence->sq_length) {
-		n = PySequence_Size(v);
-		if (n < 0)
-			PyErr_Clear();
-	}
-	if (n < 0)
-		n = 8;	/* arbitrary */
-	result = PyList_New(n);
-	if (result == NULL) {
-		Py_DECREF(it);
+	rv = _PyList_Extend((PyListObject *)result, v);
+	if (rv == NULL) {
+		Py_DECREF(result);
 		return NULL;
 	}
-
-	/* Run iterator to exhaustion. */
-	for (i = 0; ; i++) {
-		PyObject *item = PyIter_Next(it);
-		if (item == NULL) {
-			if (PyErr_Occurred()) {
-				Py_DECREF(result);
-				result = NULL;
-			}
-			break;
-		}
-		if (i < n)
-			PyList_SET_ITEM(result, i, item); /* steals ref */
-		else {
-			int status = PyList_Append(result, item);
-			Py_DECREF(item);  /* append creates a new ref */
-			if (status < 0) {
-				Py_DECREF(result);
-				result = NULL;
-				break;
-			}
-		}
-	}
-
-	/* Cut back result list if initial guess was too large. */
-	if (i < n && result != NULL) {
-		if (PyList_SetSlice(result, i, n, (PyObject *)NULL) != 0) {
-			Py_DECREF(result);
-			result = NULL;
-		}
-	}
-	Py_DECREF(it);
+	Py_DECREF(rv);
 	return result;
 }
 
 PyObject *
 PySequence_Fast(PyObject *v, const char *m)
 {
+	PyObject *it;
+
 	if (v == NULL)
 		return null_error();
 
-	if (PyList_Check(v) || PyTuple_Check(v)) {
+	if (PyList_CheckExact(v) || PyTuple_CheckExact(v)) {
 		Py_INCREF(v);
 		return v;
 	}
 
-	v = PySequence_Tuple(v);
-	if (v == NULL && PyErr_ExceptionMatches(PyExc_TypeError))
-		return type_error(m);
+ 	it = PyObject_GetIter(v);
+	if (it == NULL) {
+		if (PyErr_ExceptionMatches(PyExc_TypeError))
+			return type_error(m);
+		return NULL;
+	}
+
+	v = PySequence_Tuple(it);
+	Py_DECREF(it);
 
 	return v;
 }
@@ -1576,8 +1629,13 @@ PySequence_Index(PyObject *s, PyObject *o)
 int
 PyMapping_Check(PyObject *o)
 {
-	return o && o->ob_type->tp_as_mapping &&
-		o->ob_type->tp_as_mapping->mp_subscript;
+	if (o && PyInstance_Check(o))
+		return PyObject_HasAttrString(o, "__getitem__");
+
+	return  o && o->ob_type->tp_as_mapping &&
+		o->ob_type->tp_as_mapping->mp_subscript &&
+		!(o->ob_type->tp_as_sequence && 
+		  o->ob_type->tp_as_sequence->sq_slice);
 }
 
 int
@@ -1727,7 +1785,7 @@ PyObject_CallFunction(PyObject *callable, char *format, ...)
 			return NULL;
 		args = a;
 	}
-	retval = PyObject_CallObject(callable, args);
+	retval = PyObject_Call(callable, args, NULL);
 
 	Py_DECREF(args);
 
@@ -1774,7 +1832,7 @@ PyObject_CallMethod(PyObject *o, char *name, char *format, ...)
 		args = a;
 	}
 
-	retval = PyObject_CallObject(func, args);
+	retval = PyObject_Call(func, args, NULL);
 
 	Py_DECREF(args);
 	Py_DECREF(func);
@@ -1927,6 +1985,15 @@ abstract_issubclass(PyObject *derived, PyObject *cls)
 	if (derived == cls)
 		return 1;
 
+	if (PyTuple_Check(cls)) {
+		/* Not a general sequence -- that opens up the road to
+		   recursion and stack overflow. */
+		n = PyTuple_GET_SIZE(cls);
+		for (i = 0; i < n; i++) {
+			if (derived == PyTuple_GET_ITEM(cls, i))
+				return 1;
+		}
+	}
 	bases = abstract_get_bases(derived);
 	if (bases == NULL) {
 		if (PyErr_Occurred())
@@ -1945,13 +2012,33 @@ abstract_issubclass(PyObject *derived, PyObject *cls)
 	return r;
 }
 
-int
-PyObject_IsInstance(PyObject *inst, PyObject *cls)
+static int
+check_class(PyObject *cls, const char *error)
+{
+	PyObject *bases = abstract_get_bases(cls);
+	if (bases == NULL) {
+		/* Do not mask errors. */
+		if (!PyErr_Occurred())
+			PyErr_SetString(PyExc_TypeError, error);
+		return 0;
+	}
+	Py_DECREF(bases);
+	return -1;
+}
+
+static int
+recursive_isinstance(PyObject *inst, PyObject *cls, int recursion_depth)
 {
 	PyObject *icls;
 	static PyObject *__class__ = NULL;
 	int retval = 0;
 
+	if (__class__ == NULL) {
+		__class__ = PyString_FromString("__class__");
+		if (__class__ == NULL)
+			return -1;
+	}
+
 	if (PyClass_Check(cls) && PyInstance_Check(inst)) {
 		PyObject *inclass =
 			(PyObject*)((PyInstanceObject*)inst)->in_class;
@@ -1959,37 +2046,45 @@ PyObject_IsInstance(PyObject *inst, PyObject *cls)
 	}
 	else if (PyType_Check(cls)) {
 		retval = PyObject_TypeCheck(inst, (PyTypeObject *)cls);
+		if (retval == 0) {
+			PyObject *c = PyObject_GetAttr(inst, __class__);
+			if (c == NULL) {
+				PyErr_Clear();
+			}
+			else {
+				if (c != (PyObject *)(inst->ob_type) &&
+				    PyType_Check(c))
+					retval = PyType_IsSubtype(
+						(PyTypeObject *)c,
+						(PyTypeObject *)cls);
+				Py_DECREF(c);
+			}
+		}
 	}
 	else if (PyTuple_Check(cls)) {
-		/* Not a general sequence -- that opens up the road to
-		   recursion and stack overflow. */
 		int i, n;
 
+                if (!recursion_depth) {
+                    PyErr_SetString(PyExc_RuntimeError,
+                                    "nest level of tuple too deep");
+                    return -1;
+                }
+
 		n = PyTuple_GET_SIZE(cls);
 		for (i = 0; i < n; i++) {
-			retval = PyObject_IsInstance(
-				inst, PyTuple_GET_ITEM(cls, i));
+			retval = recursive_isinstance(
+                                    inst,
+                                    PyTuple_GET_ITEM(cls, i),
+                                    recursion_depth-1);
 			if (retval != 0)
 				break;
 		}
-		return retval;
 	}
 	else {
-		PyObject *cls_bases = abstract_get_bases(cls);
-		if (cls_bases == NULL) {
-			/* Do not mask errors. */
-			if (!PyErr_Occurred())
-				PyErr_SetString(PyExc_TypeError,
-				"isinstance() arg 2 must be a class, type,"
-				" or tuple of classes and types");
+		if (!check_class(cls,
+			"isinstance() arg 2 must be a class, type,"
+			" or tuple of classes and types"))
 			return -1;
-		}
-		Py_DECREF(cls_bases);
-		if (__class__ == NULL) {
-			__class__ = PyString_FromString("__class__");
-			if (__class__ == NULL)
-				return -1;
-		}
 		icls = PyObject_GetAttr(inst, __class__);
 		if (icls == NULL) {
 			PyErr_Clear();
@@ -2005,33 +2100,48 @@ PyObject_IsInstance(PyObject *inst, PyObject *cls)
 }
 
 int
-PyObject_IsSubclass(PyObject *derived, PyObject *cls)
+PyObject_IsInstance(PyObject *inst, PyObject *cls)
+{
+    return recursive_isinstance(inst, cls, Py_GetRecursionLimit());
+}
+
+static  int
+recursive_issubclass(PyObject *derived, PyObject *cls, int recursion_depth)
 {
 	int retval;
 
 	if (!PyClass_Check(derived) || !PyClass_Check(cls)) {
-		PyObject *derived_bases;
-		PyObject *cls_bases;
-	       
-		derived_bases = abstract_get_bases(derived);
-		if (derived_bases == NULL) {
-			/* Do not mask errors */
-			if (!PyErr_Occurred())
-				PyErr_SetString(PyExc_TypeError,
-					"issubclass() arg 1 must be a class");
+		if (!check_class(derived,
+				 "issubclass() arg 1 must be a class"))
 			return -1;
-		}
-		Py_DECREF(derived_bases);
 
-		cls_bases = abstract_get_bases(cls);
-		if (cls_bases == NULL) {
-			/* Do not mask errors */
-			if (!PyErr_Occurred())
-				PyErr_SetString(PyExc_TypeError,
-					"issubclass() arg 2 must be a class");
-			return -1;
+		if (PyTuple_Check(cls)) {
+			int i;
+			int n = PyTuple_GET_SIZE(cls);
+
+                        if (!recursion_depth) {
+                            PyErr_SetString(PyExc_RuntimeError,
+                                            "nest level of tuple too deep");
+                            return -1;
+                        }
+			for (i = 0; i < n; ++i) {
+				retval = recursive_issubclass(
+                                            derived,
+                                            PyTuple_GET_ITEM(cls, i),
+                                            recursion_depth-1);
+				if (retval != 0) {
+					/* either found it, or got an error */
+					return retval;
+				}
+			}
+			return 0;
+		}
+		else {
+			if (!check_class(cls,
+					"issubclass() arg 2 must be a class"
+					" or tuple of classes"))
+				return -1;
 		}
-		Py_DECREF(cls_bases);
 
 		retval = abstract_issubclass(derived, cls);
 	}
@@ -2044,6 +2154,13 @@ PyObject_IsSubclass(PyObject *derived, PyObject *cls)
 	return retval;
 }
 
+int
+PyObject_IsSubclass(PyObject *derived, PyObject *cls)
+{
+    return recursive_issubclass(derived, cls, Py_GetRecursionLimit());
+}
+
+
 PyObject *
 PyObject_GetIter(PyObject *o)
 {
@@ -2054,7 +2171,7 @@ PyObject_GetIter(PyObject *o)
 	if (f == NULL) {
 		if (PySequence_Check(o))
 			return PySeqIter_New(o);
-		PyErr_SetString(PyExc_TypeError, 
+		PyErr_SetString(PyExc_TypeError,
 				"iteration over non-sequence");
 		return NULL;
 	}
@@ -2083,12 +2200,7 @@ PyObject *
 PyIter_Next(PyObject *iter)
 {
 	PyObject *result;
-	if (!PyIter_Check(iter)) {
-		PyErr_Format(PyExc_TypeError,
-			     "'%.100s' object is not an iterator",
-			     iter->ob_type->tp_name);
-		return NULL;
-	}
+	assert(PyIter_Check(iter));
 	result = (*iter->ob_type->tp_iternext)(iter);
 	if (result == NULL &&
 	    PyErr_Occurred() &&
@@ -2096,4 +2208,3 @@ PyIter_Next(PyObject *iter)
 		PyErr_Clear();
 	return result;
 }
-
