@@ -12,76 +12,41 @@
 #endif
 #include <signal.h>
 
-
-/* try to determine what version of the Pthread Standard is installed.
- * this is important, since all sorts of parameter types changed from
- * draft to draft and there are several (incompatible) drafts in
- * common use.  these macros are a start, at least. 
- * 12 May 1997 -- david arnold <davida@pobox.com>
- */
-
-#if defined(__ultrix) && defined(__mips) && defined(_DECTHREADS_)
-/* _DECTHREADS_ is defined in cma.h which is included by pthread.h */
-#  define PY_PTHREAD_D4
-
-#elif defined(__osf__) && defined (__alpha)
-/* _DECTHREADS_ is defined in cma.h which is included by pthread.h */
-#  if !defined(_PTHREAD_ENV_ALPHA) || defined(_PTHREAD_USE_D4) || defined(PTHREAD_USE_D4)
-#    define PY_PTHREAD_D4
-#  else
-#    define PY_PTHREAD_STD
-#  endif
-
-#elif defined(_AIX)
-/* SCHED_BG_NP is defined if using AIX DCE pthreads
- * but it is unsupported by AIX 4 pthreads. Default
- * attributes for AIX 4 pthreads equal to NULL. For
- * AIX DCE pthreads they should be left unchanged.
- */
-#  if !defined(SCHED_BG_NP)
-#    define PY_PTHREAD_STD
-#  else
-#    define PY_PTHREAD_D7
-#  endif
-
-#elif defined(__DGUX)
-#  define PY_PTHREAD_D6
-
-#elif defined(__hpux) && defined(_DECTHREADS_)
-#  define PY_PTHREAD_D4
-
-#else /* Default case */
-#  define PY_PTHREAD_STD
-
+/* The POSIX spec says that implementations supporting the sem_*
+   family of functions must indicate this by defining
+   _POSIX_SEMAPHORES. */   
+#ifdef _POSIX_SEMAPHORES
+#include <semaphore.h>
+#include <errno.h>
 #endif
 
-#ifdef USE_GUSI
-/* The Macintosh GUSI I/O library sets the stackspace to
-** 20KB, much too low. We up it to 64K.
-*/
-#define THREAD_STACK_SIZE 0x10000
-#endif
-
-
-/* set default attribute object for different versions */
-
-#if defined(PY_PTHREAD_D4) || defined(PY_PTHREAD_D7)
-#  define pthread_attr_default pthread_attr_default
-#  define pthread_mutexattr_default pthread_mutexattr_default
-#  define pthread_condattr_default pthread_condattr_default
-#elif defined(PY_PTHREAD_STD) || defined(PY_PTHREAD_D6)
+#if !defined(pthread_attr_default)
 #  define pthread_attr_default ((pthread_attr_t *)NULL)
+#endif
+#if !defined(pthread_mutexattr_default)
 #  define pthread_mutexattr_default ((pthread_mutexattr_t *)NULL)
+#endif
+#if !defined(pthread_condattr_default)
 #  define pthread_condattr_default ((pthread_condattr_t *)NULL)
 #endif
 
 
+/* Whether or not to use semaphores directly rather than emulating them with
+ * mutexes and condition variables:
+ */
+#if defined(_POSIX_SEMAPHORES) && !defined(HAVE_BROKEN_POSIX_SEMAPHORES)
+#  define USE_SEMAPHORES
+#else
+#  undef USE_SEMAPHORES
+#endif
+
+
 /* On platforms that don't use standard POSIX threads pthread_sigmask()
  * isn't present.  DEC threads uses sigprocmask() instead as do most
  * other UNIX International compliant systems that don't have the full
  * pthread implementation.
  */
-#ifdef HAVE_PTHREAD_SIGMASK
+#if defined(HAVE_PTHREAD_SIGMASK) && !defined(HAVE_BROKEN_PTHREAD_SIGMASK)
 #  define SET_THREAD_SIGMASK pthread_sigmask
 #else
 #  define SET_THREAD_SIGMASK sigprocmask
@@ -153,8 +118,7 @@ long
 PyThread_start_new_thread(void (*func)(void *), void *arg)
 {
 	pthread_t th;
-	int success;
- 	sigset_t oldmask, newmask;
+	int status;
 #if defined(THREAD_STACK_SIZE) || defined(PTHREAD_SYSTEM_SCHED_SUPPORTED)
 	pthread_attr_t attrs;
 #endif
@@ -168,31 +132,11 @@ PyThread_start_new_thread(void (*func)(void *), void *arg)
 #ifdef THREAD_STACK_SIZE
 	pthread_attr_setstacksize(&attrs, THREAD_STACK_SIZE);
 #endif
-#ifdef PTHREAD_SYSTEM_SCHED_SUPPORTED
+#if defined(PTHREAD_SYSTEM_SCHED_SUPPORTED) && !defined(__FreeBSD__)
         pthread_attr_setscope(&attrs, PTHREAD_SCOPE_SYSTEM);
 #endif
 
-	/* Mask all signals in the current thread before creating the new
-	 * thread.  This causes the new thread to start with all signals
-	 * blocked.
-	 */
-	sigfillset(&newmask);
-	SET_THREAD_SIGMASK(SIG_BLOCK, &newmask, &oldmask);
-
-	success = pthread_create(&th, 
-#if defined(PY_PTHREAD_D4)
-				 pthread_attr_default,
-				 (pthread_startroutine_t)func, 
-				 (pthread_addr_t)arg
-#elif defined(PY_PTHREAD_D6)
-				 pthread_attr_default,
-				 (void* (*)(void *))func,
-				 arg
-#elif defined(PY_PTHREAD_D7)
-				 pthread_attr_default,
-				 func,
-				 arg
-#elif defined(PY_PTHREAD_STD)
+	status = pthread_create(&th, 
 #if defined(THREAD_STACK_SIZE) || defined(PTHREAD_SYSTEM_SCHED_SUPPORTED)
 				 &attrs,
 #else
@@ -200,22 +144,16 @@ PyThread_start_new_thread(void (*func)(void *), void *arg)
 #endif
 				 (void* (*)(void *))func,
 				 (void *)arg
-#endif
 				 );
 
-	/* Restore signal mask for original thread */
-	SET_THREAD_SIGMASK(SIG_SETMASK, &oldmask, NULL);
-
 #if defined(THREAD_STACK_SIZE) || defined(PTHREAD_SYSTEM_SCHED_SUPPORTED)
 	pthread_attr_destroy(&attrs);
 #endif
-	if (success == 0) {
-#if defined(PY_PTHREAD_D4) || defined(PY_PTHREAD_D6) || defined(PY_PTHREAD_D7)
-		pthread_detach(&th);
-#elif defined(PY_PTHREAD_STD)
-		pthread_detach(th);
-#endif
-	}
+	if (status != 0)
+            return -1;
+
+        pthread_detach(th);
+
 #if SIZEOF_PTHREAD_T <= SIZEOF_LONG
 	return (long) th;
 #else
@@ -225,7 +163,7 @@ PyThread_start_new_thread(void (*func)(void *), void *arg)
 
 /* XXX This implementation is considered (to quote Tim Peters) "inherently
    hosed" because:
-     - It does not guanrantee the promise that a non-zero integer is returned.
+     - It does not guarantee the promise that a non-zero integer is returned.
      - The cast to long is inherently unsafe.
      - It is not clear that the 'volatile' (for AIX?) and ugly casting in the
        latter return statement (for Alpha OSF/1) are any longer necessary.
@@ -294,6 +232,109 @@ PyThread__exit_prog(int status)
 }
 #endif /* NO_EXIT_PROG */
 
+#ifdef USE_SEMAPHORES
+
+/*
+ * Lock support.
+ */
+
+PyThread_type_lock 
+PyThread_allocate_lock(void)
+{
+	sem_t *lock;
+	int status, error = 0;
+
+	dprintf(("PyThread_allocate_lock called\n"));
+	if (!initialized)
+		PyThread_init_thread();
+
+	lock = (sem_t *)malloc(sizeof(sem_t));
+
+	if (lock) {
+		status = sem_init(lock,0,1);
+		CHECK_STATUS("sem_init");
+
+		if (error) {
+			free((void *)lock);
+			lock = NULL;
+		}
+	}
+
+	dprintf(("PyThread_allocate_lock() -> %p\n", lock));
+	return (PyThread_type_lock)lock;
+}
+
+void 
+PyThread_free_lock(PyThread_type_lock lock)
+{
+	sem_t *thelock = (sem_t *)lock;
+	int status, error = 0;
+
+	dprintf(("PyThread_free_lock(%p) called\n", lock));
+
+	if (!thelock)
+		return;
+
+	status = sem_destroy(thelock);
+	CHECK_STATUS("sem_destroy");
+
+	free((void *)thelock);
+}
+
+/*
+ * As of February 2002, Cygwin thread implementations mistakenly report error
+ * codes in the return value of the sem_ calls (like the pthread_ functions).
+ * Correct implementations return -1 and put the code in errno. This supports
+ * either.
+ */
+static int
+fix_status(int status)
+{
+	return (status == -1) ? errno : status;
+}
+
+int 
+PyThread_acquire_lock(PyThread_type_lock lock, int waitflag)
+{
+	int success;
+	sem_t *thelock = (sem_t *)lock;
+	int status, error = 0;
+
+	dprintf(("PyThread_acquire_lock(%p, %d) called\n", lock, waitflag));
+
+	do {
+		if (waitflag)
+			status = fix_status(sem_wait(thelock));
+		else
+			status = fix_status(sem_trywait(thelock));
+	} while (status == EINTR); /* Retry if interrupted by a signal */
+
+	if (waitflag) {
+		CHECK_STATUS("sem_wait");
+	} else if (status != EAGAIN) {
+		CHECK_STATUS("sem_trywait");
+	}
+	
+	success = (status == 0) ? 1 : 0;
+
+	dprintf(("PyThread_acquire_lock(%p, %d) -> %d\n", lock, waitflag, success));
+	return success;
+}
+
+void 
+PyThread_release_lock(PyThread_type_lock lock)
+{
+	sem_t *thelock = (sem_t *)lock;
+	int status, error = 0;
+
+	dprintf(("PyThread_release_lock(%p) called\n", lock));
+
+	status = sem_post(thelock);
+	CHECK_STATUS("sem_post");
+}
+
+#else /* USE_SEMAPHORES */
+
 /*
  * Lock support.
  */
@@ -359,27 +400,23 @@ PyThread_acquire_lock(PyThread_type_lock lock, int waitflag)
 	status = pthread_mutex_lock( &thelock->mut );
 	CHECK_STATUS("pthread_mutex_lock[1]");
 	success = thelock->locked == 0;
-	if (success) thelock->locked = 1;
-	status = pthread_mutex_unlock( &thelock->mut );
-	CHECK_STATUS("pthread_mutex_unlock[1]");
 
 	if ( !success && waitflag ) {
 		/* continue trying until we get the lock */
 
 		/* mut must be locked by me -- part of the condition
 		 * protocol */
-		status = pthread_mutex_lock( &thelock->mut );
-		CHECK_STATUS("pthread_mutex_lock[2]");
 		while ( thelock->locked ) {
 			status = pthread_cond_wait(&thelock->lock_released,
 						   &thelock->mut);
 			CHECK_STATUS("pthread_cond_wait");
 		}
-		thelock->locked = 1;
-		status = pthread_mutex_unlock( &thelock->mut );
-		CHECK_STATUS("pthread_mutex_unlock[2]");
 		success = 1;
 	}
+	if (success) thelock->locked = 1;
+	status = pthread_mutex_unlock( &thelock->mut );
+	CHECK_STATUS("pthread_mutex_unlock[1]");
+
 	if (error) success = 0;
 	dprintf(("PyThread_acquire_lock(%p, %d) -> %d\n", lock, waitflag, success));
 	return success;
@@ -406,100 +443,4 @@ PyThread_release_lock(PyThread_type_lock lock)
 	CHECK_STATUS("pthread_cond_signal");
 }
 
-/*
- * Semaphore support.
- */
-
-struct semaphore {
-	pthread_mutex_t mutex;
-	pthread_cond_t cond;
-	int value;
-};
-
-PyThread_type_sema 
-PyThread_allocate_sema(int value)
-{
-	struct semaphore *sema;
-	int status, error = 0;
-
-	dprintf(("PyThread_allocate_sema called\n"));
-	if (!initialized)
-		PyThread_init_thread();
-
-	sema = (struct semaphore *) malloc(sizeof(struct semaphore));
-	if (sema != NULL) {
-		sema->value = value;
-		status = pthread_mutex_init(&sema->mutex,
-					    pthread_mutexattr_default);
-		CHECK_STATUS("pthread_mutex_init");
-		status = pthread_cond_init(&sema->cond,
-					   pthread_condattr_default);
-		CHECK_STATUS("pthread_cond_init");
-		if (error) {
-			free((void *) sema);
-			sema = NULL;
-		}
-	}
-	dprintf(("PyThread_allocate_sema() -> %p\n",  sema));
-	return (PyThread_type_sema) sema;
-}
-
-void 
-PyThread_free_sema(PyThread_type_sema sema)
-{
-	int status, error = 0;
-	struct semaphore *thesema = (struct semaphore *) sema;
-
-	dprintf(("PyThread_free_sema(%p) called\n",  sema));
-	status = pthread_cond_destroy(&thesema->cond);
-	CHECK_STATUS("pthread_cond_destroy");
-	status = pthread_mutex_destroy(&thesema->mutex);
-	CHECK_STATUS("pthread_mutex_destroy");
-	free((void *) thesema);
-}
-
-int 
-PyThread_down_sema(PyThread_type_sema sema, int waitflag)
-{
-	int status, error = 0, success;
-	struct semaphore *thesema = (struct semaphore *) sema;
-
-	dprintf(("PyThread_down_sema(%p, %d) called\n",  sema, waitflag));
-	status = pthread_mutex_lock(&thesema->mutex);
-	CHECK_STATUS("pthread_mutex_lock");
-	if (waitflag) {
-		while (!error && thesema->value <= 0) {
-			status = pthread_cond_wait(&thesema->cond,
-						   &thesema->mutex);
-			CHECK_STATUS("pthread_cond_wait");
-		}
-	}
-	if (error)
-		success = 0;
-	else if (thesema->value > 0) {
-		thesema->value--;
-		success = 1;
-	}
-	else
-		success = 0;
-	status = pthread_mutex_unlock(&thesema->mutex);
-	CHECK_STATUS("pthread_mutex_unlock");
-	dprintf(("PyThread_down_sema(%p) return\n",  sema));
-	return success;
-}
-
-void 
-PyThread_up_sema(PyThread_type_sema sema)
-{
-	int status, error = 0;
-	struct semaphore *thesema = (struct semaphore *) sema;
-
-	dprintf(("PyThread_up_sema(%p)\n",  sema));
-	status = pthread_mutex_lock(&thesema->mutex);
-	CHECK_STATUS("pthread_mutex_lock");
-	thesema->value++;
-	status = pthread_cond_signal(&thesema->cond);
-	CHECK_STATUS("pthread_cond_signal");
-	status = pthread_mutex_unlock(&thesema->mutex);
-	CHECK_STATUS("pthread_mutex_unlock");
-}
+#endif /* USE_SEMAPHORES */
